#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "gp2x.h"
#include <SDL_gp2x.h>
#include <Vector>
#include <SDL_gfxPrimitives.h>
#include <SDL/SDL_gfxPrimitives.h>

using namespace std;


enum GameStates {GSQuit,GSIntro,GSGame,GSTitleScreen,GSWinner};

struct SBlock{
   bool up,left,down,right ;
   int blocknr;
};

struct SMazeRoom{
    bool NorthWall,SouthWall,WestWall,EastWall,visited;
};

struct SPoint{
    int X,Y;
};

const int
    MaxMazeWidth = 63,
    MaxMazeHeight = 47;
//Screen is het scherm, Buffer is de groote van het spelscherm (altijd 320x240)
//Buffer1 is de groote van het spel scherm of de groote de tv out mode
// er wordt dus altijd naar Buffer getekend, buffer wordt naar buffer1 geblit en buffer1 naar het scherm
SDL_Surface *Buffer,*Buffer1,*Screen;
TTF_Font* font;
SDL_Joystick *Joystick;
bool GlobalSoundEnabled = true,TvOutMode=false;
GameStates GameState = GSTitleScreen;
int Volume = 128,StartScreenX=0,StartScreenY=0,InitialStartScreenX=20,InitialStartScreenY=24,BoxWidth=5,BoxHeight=5,MazeWidth=10,MazeHeight=10, PlayerPosX=62,PlayerPosY=46;
SMazeRoom Maze[MaxMazeWidth][MaxMazeHeight];

// set the volume of the music and both channels (left/right)
void SetVolume(const int VolumeIn)
{
	Mix_Volume(0,VolumeIn);
	Mix_Volume(1,VolumeIn);
	Mix_VolumeMusic(VolumeIn);
}

// increase the volume with 8
void IncVolume()
{
	if (Volume < 128)
	{
		Volume += 8;
		SetVolume(Volume);
	}
}

// decrease the volume with 8
void DecVolume()
{
	if (Volume > 0)
	{
		Volume-= 8;
		SetVolume(Volume);
	}
}

void LoadSettings()
{
 	FILE *SettingsFile;
 	int Value=1;
 	SettingsFile = fopen("./settings.dat","r");
 	if(SettingsFile)
 	{
		fscanf(SettingsFile,"Volume=%d\n",&Volume);
		SetVolume(Volume);
		if(TvOutMode)
		{
            fscanf(SettingsFile,"StartScreenX=%d\n",&StartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&StartScreenY);
		}
		else
        {
            fscanf(SettingsFile,"StartScreenX=%d\n",&InitialStartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&InitialStartScreenY);
            StartScreenX = 0;
            StartScreenY = 0;
        }
		fclose(SettingsFile);
 	}
 	else
 	{
		SetVolume(128);
		StartScreenX = (Screen->w - Buffer->w) / 2;
        StartScreenY = (Screen->h - Buffer->h) / 2;
	}
}

void SaveSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","w");
 	if(SettingsFile)
 	{
		fprintf(SettingsFile,"Volume=%d\n",Volume);
        if(TvOutMode)
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",StartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",StartScreenY);
        }
        else
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",InitialStartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",InitialStartScreenY);
        }
		fclose(SettingsFile);
		sync();
 	}
}


void WriteText(SDL_Surface* Surface,TTF_Font* FontIn,char* Tekst,int NrOfChars,int X,int Y,int YSpacing,SDL_Color ColorIn)
{
	char List[100][255];
	int Lines,Teller,Chars;
	SDL_Rect DstRect;
	SDL_Surface *TmpSurface1,*TmpSurface2;
	Lines = 0;
	Chars = 0;
	for (Teller=0;Teller<NrOfChars;Teller++)
	{
		if(Lines > 100)
			break;
		if((Tekst[Teller] == '\n') || (Chars==255))
		{
			List[Lines][Chars]='\0';
			Lines++;
			Chars = 0;
		}
		else
		{
		 	List[Lines][Chars]=Tekst[Teller];
		 	Chars++;
		}
	}
	List[Lines][Chars] = '\0';
	for (Teller=0;Teller <= Lines;Teller++)
	{
		if(strlen(List[Teller]) > 0)
		{
			TmpSurface1 = TTF_RenderText_Blended(FontIn,List[Teller],ColorIn);
			if (TmpSurface1)
			{
				DstRect.x = X;
				DstRect.y = Y + (Teller) * TTF_FontLineSkip(FontIn) + (Teller*YSpacing);
				DstRect.w = TmpSurface1->w;
				DstRect.h = TmpSurface1->h;
				SDL_BlitSurface(TmpSurface1,NULL,Surface,&DstRect);
				SDL_FreeSurface(TmpSurface1);
			}
		}
	}
}


void DrawMaze(SDL_Surface *Surface)
{
    int X,Y;
    SDL_FillRect(Surface,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
    for (Y= 0;Y< MazeHeight;Y++)
        for(X = 0;X < MazeWidth;X++)
        {
            if (Maze[X][Y].NorthWall)
            {
                lineRGBA(Surface,(X) * BoxWidth,(Y) * BoxHeight,((X+1)*BoxWidth),((Y)*BoxHeight),255,255,255,255);
            }

            if(Maze[X][Y].SouthWall)
            {
                lineRGBA(Surface,((X) * BoxWidth),((Y+1) * BoxHeight),((X+1)*BoxWidth),((Y+1)*BoxHeight),255,255,255,255);
            }


            if(Maze[X][Y].WestWall)
            {
                lineRGBA(Surface,((X) * BoxWidth),(Y) * BoxHeight,((X)*BoxWidth),((Y+1)*BoxHeight),255,255,255,255);
            }

            if(Maze[X][Y].EastWall)
            {
                lineRGBA(Surface,((X+1) * BoxWidth),(Y) * BoxHeight,((X+1)*BoxWidth),((Y+1)*BoxHeight),255,255,255,255);
            }
        }
}

void InitMaze()
{
    int x,y;
    for (y=0;y <MaxMazeHeight;y++)
        for (x=0;x < MaxMazeWidth;x++)
        {
            Maze[x][y].NorthWall = true;
            Maze[x][y].SouthWall = true;
            Maze[x][y].EastWall = true;
            Maze[x][y].WestWall = true;
            Maze[x][y].visited = false;
        }
}

void GenerateMaze()
{
    int Teller,TotalRooms,VisitedRooms,NeighboursFound,SelectedNeighbour;
    vector<SPoint> CellStack;
    SPoint CurrentPoint;
    SPoint Neighbours[4];
    InitMaze();
    TotalRooms = MazeWidth * MazeHeight;
    CurrentPoint.X =rand() % MazeWidth;
    CurrentPoint.Y =rand() % MazeHeight;
    VisitedRooms = 1;
    while (VisitedRooms < TotalRooms)
    {
        NeighboursFound = 0;
        //left of current room
        if (CurrentPoint.X +1 < MazeWidth)
            if (Maze[CurrentPoint.X +1][CurrentPoint.Y].NorthWall &&
                Maze[CurrentPoint.X +1][CurrentPoint.Y].SouthWall &&
                Maze[CurrentPoint.X +1][CurrentPoint.Y].WestWall &&
                Maze[CurrentPoint.X +1][CurrentPoint.Y].EastWall)
            {
                Neighbours[NeighboursFound].X = CurrentPoint.X +1;
                Neighbours[NeighboursFound].Y = CurrentPoint.Y  ;
                NeighboursFound++;
            }

        if (CurrentPoint.X -1 >=0)
            if (Maze[CurrentPoint.X -1][CurrentPoint.Y].NorthWall &&
                Maze[CurrentPoint.X -1][CurrentPoint.Y].SouthWall &&
                Maze[CurrentPoint.X -1][CurrentPoint.Y].WestWall &&
                Maze[CurrentPoint.X -1][CurrentPoint.Y].EastWall)
            {
                Neighbours[NeighboursFound].X = CurrentPoint.X -1;
                Neighbours[NeighboursFound].Y = CurrentPoint.Y  ;
                NeighboursFound++;
            }

        if (CurrentPoint.Y -1 >= 0)
            if (Maze[CurrentPoint.X ][CurrentPoint.Y-1].NorthWall &&
                Maze[CurrentPoint.X ][CurrentPoint.Y-1].SouthWall &&
                Maze[CurrentPoint.X ][CurrentPoint.Y-1].WestWall &&
                Maze[CurrentPoint.X ][CurrentPoint.Y-1].EastWall)
            {
                Neighbours[NeighboursFound].X = CurrentPoint.X ;
                Neighbours[NeighboursFound].Y = CurrentPoint.Y -1;
                NeighboursFound++;
            }

        if (CurrentPoint.Y + 1 < MazeHeight)
            if (Maze[CurrentPoint.X ][CurrentPoint.Y+1].NorthWall &&
                Maze[CurrentPoint.X ][CurrentPoint.Y+1].SouthWall &&
                Maze[CurrentPoint.X ][CurrentPoint.Y+1].WestWall &&
                Maze[CurrentPoint.X ][CurrentPoint.Y+1].EastWall)
            {
                Neighbours[NeighboursFound].X = CurrentPoint.X ;
                Neighbours[NeighboursFound].Y = CurrentPoint.Y +1 ;
                NeighboursFound++;
            }

        if (NeighboursFound > 0)
        {
            SelectedNeighbour = rand() % NeighboursFound;

            // neighbour to the east
            if( Neighbours[SelectedNeighbour].X > CurrentPoint.X)
            {
                Maze[Neighbours[SelectedNeighbour].X][Neighbours[SelectedNeighbour].Y].WestWall = false;
                Maze[CurrentPoint.X][CurrentPoint.Y].EastWall = false;
            }
            else // neighbour to the west
                if( Neighbours[SelectedNeighbour].X < CurrentPoint.X)
                {
                    Maze[Neighbours[SelectedNeighbour].X][Neighbours[SelectedNeighbour].Y].EastWall  = false;
                    Maze[CurrentPoint.X][CurrentPoint.Y].WestWall = false;
                }
                else // neighbour to the north
                    if(Neighbours[SelectedNeighbour].Y < CurrentPoint.Y)
                    {
                        Maze[Neighbours[SelectedNeighbour].X][Neighbours[SelectedNeighbour].Y].SouthWall  = false;
                        Maze[CurrentPoint.X][CurrentPoint.Y].NorthWall = false;
                    }
                    else // neighbour to the south
                        if (Neighbours[SelectedNeighbour].Y > CurrentPoint.Y)
                        {
                            Maze[Neighbours[SelectedNeighbour].X][Neighbours[SelectedNeighbour].Y].NorthWall  = false;
                            Maze[CurrentPoint.X][CurrentPoint.Y].SouthWall  = false;
                        }

            CellStack.push_back(CurrentPoint);
            CurrentPoint.X =Neighbours[SelectedNeighbour].X;
            CurrentPoint.Y =Neighbours[SelectedNeighbour].Y;
            VisitedRooms = VisitedRooms + 1;
        }
        else
        {
            CurrentPoint = CellStack.back();
            CellStack.pop_back();
        }
    }
    CellStack.clear();
    Maze[0][0].NorthWall = false;
    Maze[MazeWidth-1][MazeHeight-1].SouthWall = false;
    PlayerPosX = MazeWidth -1;
    PlayerPosY = MazeHeight - 1;
}

void TitleScreen()
{
    SDL_Rect Rect;
    SDL_Event Event;
    int Selection = 1;
    Uint32 Time = 0;
    bool RPressed = false,LeftPressed = false,RightPressed = false;
    char Text[500];
    SDL_Color Color1={255,255,255,255},Color2={100,100,255,255};
    Time = SDL_GetTicks();
    while (GameState == GSTitleScreen)
    {
        while (SDL_PollEvent(&Event))
		{
		    if (Event.type == SDL_JOYBUTTONUP)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_R:
                        RPressed = false;
                        break;
                    case GP2X_BUTTON_LEFT:
                        LeftPressed =false;
                        if (RPressed)
                            if(StartScreenX - 2 >= 0)
                                StartScreenX -=2;
                        break;
                    case GP2X_BUTTON_RIGHT:
                        RightPressed = false;
                        if (RPressed)
                            if(StartScreenX + 2 + Buffer->w <= Screen->w)
                                StartScreenX +=2;
                        break;
                    case GP2X_BUTTON_UP:
                        if(RPressed)
                            if(StartScreenY - 2 >=0)
                                StartScreenY -=2;
                        break;
                    case GP2X_BUTTON_DOWN:
                        if(RPressed)
                            if(StartScreenY + 2 + Buffer->h <= Screen->h)
                                StartScreenY +=2;
                        break;
                }

            if (Event.type == SDL_JOYBUTTONDOWN)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_LEFT:
                        LeftPressed =true;
                        break;
                    case GP2X_BUTTON_RIGHT:
                        RightPressed = true;
                        break;
                    case GP2X_BUTTON_R:
                        RPressed = true;
                        break;
                    case GP2X_BUTTON_SELECT:
                        GameState= GSQuit;
                        break;
                    case GP2X_BUTTON_UP:
                        if (Selection > 1)
                            Selection--;
                        break;
                    case GP2X_BUTTON_DOWN:
                        if (Selection < 4)
                            Selection++;
                        break;
                    case GP2X_BUTTON_A:
                    case GP2X_BUTTON_B:
                    case GP2X_BUTTON_X:
                    case GP2X_BUTTON_Y:
                    case GP2X_BUTTON_START:
                        switch (Selection)
                        {
                            case 1 :
                                GameState = GSGame;
                                break;
                            case 4:
                                GameState = GSQuit;
                                break;
                        }
                }
        }
        SDL_FillRect(Buffer,NULL,SDL_MapRGB(Buffer->format,0,0,0));
        WriteText(Buffer,font,"MAZE THINGIE",12,110,30,0,Color1);
        if (Selection==1)
            WriteText(Buffer,font,"Play",4,120,80,0,Color2);
        else
            WriteText(Buffer,font,"Play",4,120,80,0,Color1);
        sprintf(Text,"Maze Width:%d",MazeWidth);

        if(Selection == 2)
            WriteText(Buffer,font,Text,strlen(Text),120,100,0,Color2);
        else
            WriteText(Buffer,font,Text,strlen(Text),120,100,0,Color1);

        sprintf(Text,"Maze Height:%d",MazeHeight);
        if(Selection == 3)
            WriteText(Buffer,font,Text,strlen(Text),120,120,0,Color2);
        else
            WriteText(Buffer,font,Text,strlen(Text),120,120,0,Color1);

        if (Selection==4)
            WriteText(Buffer,font,"Quit",4,120,140,0,Color2);
        else
            WriteText(Buffer,font,"Quit",4,120,140,0,Color1);

        WriteText(Buffer,font,"Created by Willems Davy\nHttp://www.willemssoft.be",69,80,190,0,Color1);
        //tekenen naar buffer
        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        if (Time+100<SDL_GetTicks())
        {
            Time = SDL_GetTicks();
            if((Selection == 2))
            {
                if (RightPressed)
                    if (MazeWidth < MaxMazeWidth)
                        MazeWidth++;
                if (LeftPressed)
                    if(MazeWidth > 10 )
                        MazeWidth--;
            }

            if((Selection == 3))
            {
                if (RightPressed)
                    if (MazeHeight < MaxMazeHeight)
                        MazeHeight++;
                if (LeftPressed)
                    if(MazeHeight > 10 )
                        MazeHeight--;
            }

        }
    }
}

void DrawPlayer(SDL_Surface *Surface)
{
    boxRGBA(Surface, PlayerPosX * BoxWidth+1, PlayerPosY * BoxHeight+1, PlayerPosX*BoxWidth + BoxWidth-1, PlayerPosY * BoxHeight + BoxHeight-1,255,0,0,255);
}

void Game()
{
    SDL_Rect Rect;
    SDL_Event Event;
    Uint32 Time = 0;
    bool GameWon = false,DirLeft=false,DirRight=false,DirUp=false,DirDown=false;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    GenerateMaze();
    Time = SDL_GetTicks();
    while (GameState == GSGame)
    {
        while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_SELECT:
						GameState = GSTitleScreen;
						break;
                    case GP2X_BUTTON_RIGHT:
                        DirRight = true;
                        break;
                    case GP2X_BUTTON_LEFT:
                        DirLeft = true;
                        break;
                    case GP2X_BUTTON_UP:
                        DirUp = true;
                        break;
                    case GP2X_BUTTON_DOWN:
                        DirDown = true;
                        break;



                    case GP2X_BUTTON_A:
                    case GP2X_BUTTON_B:
                    case GP2X_BUTTON_X:
                    case GP2X_BUTTON_Y:
						GenerateMaze();
						break;
                    case GP2X_BUTTON_START:
                        PlayerPosX = MazeWidth -1;
                        PlayerPosY = MazeHeight -1;
                        break;
				}
			}

			if (Event.type == SDL_JOYBUTTONUP)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_SELECT:
						GameState = GSQuit;
						break;
                    case GP2X_BUTTON_RIGHT:
                        DirRight = false;
                        break;
                    case GP2X_BUTTON_LEFT:
                        DirLeft = false;
                        break;
                    case GP2X_BUTTON_UP:
                        DirUp = false;
                        break;
                    case GP2X_BUTTON_DOWN:
                        DirDown = false;
                        break;
				}
			}

		}
		if (Time +100 < SDL_GetTicks())
		{
		    Time = SDL_GetTicks();
            if (DirRight)
                if (!Maze[PlayerPosX][PlayerPosY].EastWall)
                    PlayerPosX++;
            if (DirLeft)
                if (!Maze[PlayerPosX][PlayerPosY].WestWall)
                    PlayerPosX--;
            if(DirUp)
                if((PlayerPosX == 0) && (PlayerPosY == 0))
                    GameWon = true;
                else
                    if (!Maze[PlayerPosX][PlayerPosY].NorthWall)
                        PlayerPosY--;
            if (DirDown)
                if((PlayerPosX != MazeWidth-1) || (PlayerPosY !=MazeHeight -1))
                    if (!Maze[PlayerPosX][PlayerPosY].SouthWall)
                        PlayerPosY++;
		}
        DrawMaze(Buffer);
        DrawPlayer(Buffer);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);

        if (GameWon)
            GameState = GSWinner;

    }
}

void Winner()
{
    SDL_Rect Rect;
    SDL_Event Event;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    char Text [500];
    GenerateMaze();
    SDL_Color Color = {255,255,255,255};
    while (GameState == GSWinner)
    {
        while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_A:
                    case GP2X_BUTTON_B:
                    case GP2X_BUTTON_X:
                    case GP2X_BUTTON_Y:
                    case GP2X_BUTTON_START:
                        GameState = GSGame;
                        break;
                    case GP2X_BUTTON_SELECT:
                        GameState = GSQuit;
                        break;
				}
			}
		}
		SDL_FillRect(Buffer,NULL,SDL_MapRGB(Buffer->format,0,0,0));
		sprintf(Text,"Conratulations!!!\nYou solved the maze,\nlet's try another one ...");
        WriteText(Buffer,font,Text,strlen(Text),75,95,0,Color);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);

    }
}


int main(int argc, char **argv)
{
    SDL_Rect Rect;
	//mtrace();
	if (SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO ) == 0)
	{
		printf("SDL Succesfully initialized\n");

        if(SDL_GP2X_GetPhysicalScreenSize(&Rect) == 0)
            Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_HWSURFACE );
        else
        {
            TvOutMode = true;
            Screen = SDL_SetVideoMode( 360, 288,16, SDL_HWSURFACE );
        }
		if(Screen)
		{
		    printf("Succesfully Set %dx%dx16\n",WINDOW_WIDTH,WINDOW_HEIGHT);
		    Buffer = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			if (TvOutMode)
                Buffer1 = SDL_CreateRGBSurface(SDL_HWSURFACE,360,288,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
            else
                Buffer1 = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			if (Buffer && Buffer1)
			{
                printf("Succesfully Created Buffer\n");
                SDL_ShowCursor(SDL_DISABLE);
                Joystick = SDL_JoystickOpen(0);
                SDL_JoystickEventState(SDL_ENABLE);
                if(Joystick)
                {
                    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,64) < 0)
                    {
                        GlobalSoundEnabled = false;
                        printf("Failed to initialise sound!\n");
                        printf("%s\n",Mix_GetError());
                    }
                    else
                    {
                        printf("Audio Succesfully initialised!\n");
                    }
                    if (TTF_Init() == 0)
                    {
                        printf("Succesfully initialized TTF\n");
                        font = TTF_OpenFont("./font.ttf",25);
                        if (font)
                        {
                            printf("Succesfully Loaded fonts\n");
                            TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
                            // Main game loop that loops untile the gamestate = GSQuit
                            // and calls the procedure according to the gamestate.
                            LoadSettings();
                            srand(time(NULL));
                            while (GameState != GSQuit)
                            {
                                switch(GameState)
                                {
                                    case GSGame :
                                        Game();
                                        break;
                                    case GSTitleScreen:
                                        TitleScreen();
                                        break;
                                    case GSWinner:
                                        Winner();
                                        break;
                                    case GSIntro :
                                        GameState=GSQuit;
                                        break;
                                }
                            }
                            SaveSettings();
                            Mix_CloseAudio();
                            TTF_CloseFont(font);
                            font=NULL;
                        }
                        else
                        {
                            printf("Failed to Load fonts\n");
                        }
                        TTF_Quit();
                    }
                    else
                    {
                        printf("Failed to initialize TTF\n");
                    }
                    if(SDL_JoystickOpened(0))
    					SDL_JoystickClose(Joystick);
                }
                else
                {
                    printf("Failed to open joystick(0)!\n");
                }
                SDL_FreeSurface(Buffer);
                Buffer=NULL;
			}
			else
                printf("Failed To Create Buffer\n");
            SDL_FreeSurface(Screen);
            Screen=NULL;
		}
		else
		{
			printf("Failed to Set Videomode %dx%dx16\n",WINDOW_WIDTH, WINDOW_HEIGHT);
		}
		SDL_Quit();
	}
	else
	{
		printf("Couldn't initialise SDL!\n");
	}
	// Shouldn't be needed but i just placed it again to be sure all the boardparts are free
	// Call the menu
	//chdir("/usr/gp2x");
	//execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);

	return 0;

}
