#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <time.h>
#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include <SDL_Gp2x.h>
#include "gp2x.h"

using namespace std;

enum GameStates {GSTitleScreen,GSOldManSpeaking,GSPasswordEntry,GSOptions,GSCredits,GSIntro,GSQuit,GSGame,GSStageClear,GSNextStage,GSStageSelect};
enum PlayerStates {Walking,Waiting,LookingUp,EnterBuilding,EnteredBuilding};
enum CloudStyles {Big,Small};

class CPlayer
{
 public:
 	CPlayer(const int Xin, const int Yin,const int MinXin,const int MaxXin);
 	void Move();
 	void Draw();
 	int GetX();
 	int GetY();
 	int GetWidth();
 	~CPlayer();
 	PlayerStates State;
 private:
 	SDL_Surface *Image,*Shadow;
 	int X,Y,AnimPhase,AnimCounter,Delay,Width,Height,MinX,MaxX;
};

class CSpaceShip
{
 public:
 	CSpaceShip();
 	void Move();
 	void Draw();
 	int GetX();
 	~CSpaceShip();
 private:
 	int X,Y;
 	SDL_Surface* Image;
};

class CCloud
{
 public:
 	CCloud(const int XIn,const int YIn,const float XiIn,CloudStyles Style);
	void Move();
	void Draw();
	~CCloud();
 private:
 	float X,Xi;
 	int Y,Width,Height;
 	SDL_Surface* Image;
};

class CFairy
{
 public:
 	CFairy(const int XIn,const int YIn,const int AnimDelay);
 	void Draw();
 	int GetX();
 	~CFairy();
 	bool Hidden;
 private:
 	int X,Y,Width,Height,Delay,AnimPhase,AnimDelay;
 	SDL_Surface* Image;
};

class CStageBlock
{
 public:
 	CStageBlock();
 	void Move();
 	void Draw();
 	int GetHeight();
 	int GetY();
 	void Load(const int BlockNr);
 	~CStageBlock();
 	bool Hidden;
 private:
 	int X,Y,Yi,Width,Height;

 	SDL_Surface* Image;
};


class CHand
{
 public:
 	CHand();
 	void Move();
 	void Draw();
 	void SetPosition(const int XIn, const int YIn);
 	void Hide();
 	void Show();
 	int GetPlayFieldX();
 	int GetPlayFieldY();
 	~CHand();
 private:
 	int X,Y;
 	bool Hidden;
 	SDL_Surface *Image;
};

class CTitleScreenSelector
{
 public:
 	CTitleScreenSelector();
 	void MoveUp();
 	void MoveDown();
 	void Draw();
 	~CTitleScreenSelector();
 	int GetSelection();
 private:
	int X,Selection;
	SDL_Surface *Image;
};

class CPasswordSelector
{
 public:
 	CPasswordSelector();
 	void MoveUp();
 	void MoveDown();
 	void MoveLeft();
 	void MoveRight();
 	void Draw();
 	int GetX();
 	int GetY();
 	~CPasswordSelector();
 private:
 	SDL_Surface* Image;
 	int X,Y;
};

class CStageSelectSelector
{
 public:
 	CStageSelectSelector();
 	void MoveLeft();
 	void MoveRight();
 	void MoveUp();
 	void MoveDown();
 	void Draw();
 	int GetSelection();
 	~CStageSelectSelector();
 private:
 	int X,Y;
 	SDL_Surface* Image;
};

class COptionsSelector
{
 public:
 	COptionsSelector();
 	void MoveUp();
 	void MoveDown();
 	void Draw();
 	int GetSelection();
 	~COptionsSelector();
 private:
 	SDL_Surface* Image;
 	int Selection;
};


const SDL_Color Color = {255,255,255,0};
//Player anims
const int AnimLeft[6]= {12,11,10,9,8,7};
const int AnimRight[6]  = {1,2,3,4,5,6};
const int AnimEnterBuilding[4]  = {15,16,17,18};

//audio
const int NROFSOUNDS= 10,
          SND_RotateBlock = 0,
          SND_DropBlock = 1,
          SND_FlipBlock = 2,
          SND_PickupBlock = 3,
          SND_Error = 4,
          SND_Menu = 5,
          SND_Select = 6,
          SND_Bridge = 7,
          SND_StageEnd = 8,
          SND_Elf = 9,

  		  NROFMUSIC = 5,
  		  MUS_Title = 0,
  		  MUS_Game1 = 1,
  		  MUS_Game2 = 2,
  		  MUS_Oldman = 3,
  		  MUS_Stage = 4,

  		  BlockCount = 13,
  		  BorderCount = 7,
  		  BlockWidth = 12,
  		  BlockHeight = 12,
  		  XOffsetGame = 40,
  		  YOffsetGame = 34,
  		  XOffsetPassword = 50,
  		  YOffsetPassword = 75,
  		  XOffsetStageSelect = 50,
  		  YOffsetStageSelect = 40,
  		  Cols = 20,
  		  Rows = 16,
  		  MinPlayAreaX = XOffsetGame,
  		  MinPlayAreaY = YOffsetGame,
  		  MaxPlayAreaX = XOffsetGame + BlockWidth * Cols,
  		  MaxPlayAreaY = YOffsetGame + BlockHeight * Rows,
  		  LetterRows = 4,
  		  LetterCols = 7;
const char* const LevelPasswords[36] = {"DAVY","DISK","TREE","SEAL",
									 "OPEN","LION","WINE","GROW",
									 "FOOD","DUEL","MEAT","FLEE",
									 "DUST","LOST","HELP","MILK",
									 "SAFE","DULL","SAVE","BEER",
									 "BILL","BLOW","BIKE","VENT",
									 "ROOM","HOOD","MOOD","FEAR",
									 "REEF","POLE","BEEF","SONG",
									 "HOPE","BLUE","MAIL","MONK"};
const char* const  Letters[4][7] = {{"A","B","C","D","E","F","G"},
											  {"H","I","J","K","L","M","N"},
											  {"O","P","Q","R","S","T","U"},
											  {"V","W","X","Y","Z","0","0"}};
    //Game Stuff
    GameStates GameState = GSIntro;
    SDL_Surface* Screen,*Buffer;
    int Level;
    TTF_Font* font;
	SDL_Joystick *Joystick;
    CHand Hand;
    time_t StartTime,EndTime;
    int StartScreenX=20,StartScreenY=24,InitialStartScreenX=20,InitialStartScreenY=24;
    bool TvOutMode=false;

    //Audio
    bool GlobalSoundEnabled = true;
    bool SoundEnabled=true,MusicEnabled=true;
    int Volume=128;
    Mix_Chunk* Sounds[NROFSOUNDS];
	Mix_Music* Music[NROFMUSIC];

    //Puzzle Game Stuff
    SDL_Surface *BlockImages[BlockCount];
    SDL_Surface *BorderImages[BorderCount];
    int PlayField[2][Cols][Rows];
    bool BlockActive;

    //stage Clear Stuff
    SDL_Surface *RoomBackground,*StageClearKader;

    //Next Stage Stuff
    SDL_Surface *Bridge;

    char chrdebug[100];

void DebugPrint()
{
	int X,Y,Layers;
	for(Layers=0;Layers<2;Layers++)
						{
							printf("Layer:%d\n",Layers);
							for(X=0;X<Cols;X++)
							{
								for(Y=0;Y<Rows;Y++)
								{
									printf("%d ",PlayField[Layers][X][Y]);
								}
								printf("\n");
							}
							printf("\n");
						}


}

void SetVolume(const int VolumeIn)
{
	Mix_Volume(0,VolumeIn);
	Mix_Volume(1,VolumeIn);
	Mix_VolumeMusic(VolumeIn);
}

void IncVolume()
{
	if (Volume < 128)
	{
		Volume += 8;
		SetVolume(Volume);
	}
}

void DecVolume()
{
	if (Volume > 0)
	{
		Volume-= 8;
		SetVolume(Volume);
	}
}




void LoadSettings()
{
 	FILE *SettingsFile;
 	int Value=1;
 	SettingsFile = fopen("./settings.dat","r");
 	if(SettingsFile)
 	{
		fscanf(SettingsFile,"SoundOn=%d\n",&Value);
		if (Value > 0)
			SoundEnabled = true;
		else
			SoundEnabled = false;
		fscanf(SettingsFile,"MusicOn=%d\n",&Value);
		if (Value > 0)
			MusicEnabled = true;
		else
			MusicEnabled = false;
		fscanf(SettingsFile,"Volume=%d\n",&Volume);
		if(TvOutMode)
		{
            fscanf(SettingsFile,"StartScreenX=%d\n",&StartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&StartScreenY);
		}
		else
        {
            fscanf(SettingsFile,"StartScreenX=%d\n",&InitialStartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&InitialStartScreenY);
            StartScreenX = 0;
            StartScreenY = 0;
        }
		SetVolume(Volume);
		fclose(SettingsFile);
 	}
 	else
 	{
 		SoundEnabled = true;
		MusicEnabled = true;
		SetVolume(128);
		StartScreenX = (Screen->w - Buffer->w) / 2;
        StartScreenY = (Screen->h - Buffer->h) / 2;
	}
}

void SaveSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","w");
 	if(SettingsFile)
 	{
		if(SoundEnabled)
			fprintf(SettingsFile,"SoundOn=%d\n",1);
		else
			fprintf(SettingsFile,"SoundOn=%d\n",0);
		if (MusicEnabled)
			fprintf(SettingsFile,"MusicOn=%d\n",1);
		else
			fprintf(SettingsFile,"MusicOn=%d\n",0);
		fprintf(SettingsFile,"Volume=%d\n",Volume);
		if(TvOutMode)
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",StartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",StartScreenY);
        }
        else
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",InitialStartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",InitialStartScreenY);
        }
		fclose(SettingsFile);
 	}
}

void WriteText(SDL_Surface* Surface,TTF_Font* FontIn,char* Tekst,int NrOfChars,int X,int Y,int YSpacing,SDL_Color ColorIn)
{
	char List[100][255];
	int Lines,Teller,Chars;
	SDL_Rect DstRect;
	SDL_Surface* TmpSurface;
	Lines = 0;
	Chars = 0;
	for (Teller=0;Teller<NrOfChars;Teller++)
	{
		if(Lines > 100)
			break;
		if((Tekst[Teller] == '\n') || (Chars==255))
		{
			List[Lines][Chars]='\0';
			Lines++;
			Chars = 0;
		}
		else
		{
		 	List[Lines][Chars]=Tekst[Teller];
		 	Chars++;
		}
	}
	List[Lines][Chars] = '\0';
	for (Teller=0;Teller <= Lines;Teller++)
	{
		if(strlen(List[Teller]) > 0)
		{
			TmpSurface = TTF_RenderText_Solid(FontIn,List[Teller],ColorIn);
			DstRect.x = X;
			DstRect.y = Y + (Teller) * TTF_FontLineSkip(FontIn) + (Teller*YSpacing);
			DstRect.w = TmpSurface->w;
			DstRect.h = TmpSurface->h;
			SDL_BlitSurface(TmpSurface,NULL,Surface,&DstRect);
			SDL_FreeSurface(TmpSurface);
		}
	}
}

void MoveBlockRight()
{
	int X,Y;
	bool CanMove;
	CanMove = true;
	for (Y=0;Y<Rows;Y++)
		if(PlayField[1][Cols-1][Y] != 0)
		{
			CanMove = false;
			break;
		}
	if (CanMove)
		for(X=Cols-1;X >= 1;X--)
			for(Y=0;Y<Rows;Y++)
				PlayField[1][X][Y] = PlayField[1][X-1][Y];
	for (Y=0;Y<Rows;Y++)
		PlayField[1][0][Y] = 0;
}

void MoveBlockLeft()
{
	int X,Y;
	bool CanMove;
	CanMove = true;
	for (Y=0;Y<Rows;Y++)
		if(PlayField[1][0][Y] != 0)
		{
			CanMove = false;
			break;
		}
	if (CanMove)
		for(X=0;X <Cols-1;X++)
			for(Y=0;Y<Rows;Y++)
				PlayField[1][X][Y] = PlayField[1][X+1][Y];
	for (Y=0;Y<Rows;Y++)
		PlayField[1][Cols-1][Y] = 0;
}

void MoveBlockUp()
{
	int X,Y;
	bool CanMove;
	CanMove = true;
	for (X=0;X<Cols;X++)
		if(PlayField[1][X][0] != 0)
		{
			CanMove = false;
			break;
		}
	if (CanMove)
		for(X=0;X<Cols;X++)
			for(Y=0;Y<Rows-1;Y++)
				PlayField[1][X][Y] = PlayField[1][X][Y+1];
	for (X=0;X<Cols;X++)
		PlayField[1][X][Rows-1] = 0;
}

void MoveBlockDown()
{
	int X,Y;
	bool CanMove;
	CanMove = true;
	for (X=0;X<Cols;X++)
		if(PlayField[1][X][Rows-1] != 0)
		{
			CanMove = false;
			break;
		}
	if (CanMove)
		for(X=0;X<Cols;X++)
			for(Y=Rows-1;Y>=1;Y--)
				PlayField[1][X][Y] = PlayField[1][X][Y-1];
	for (X=0;X<Cols;X++)
		PlayField[1][X][0] = 0;
}

void MakeBlockActive(const int X,const int Y,const int BlockNr)
{
	int TellerX,TellerY;
	if((X >= 0) && (Y >=0) && (X < Cols) && (Y < Rows))
	{

		for(TellerX=Cols-1;TellerX>=0;TellerX--)
			for(TellerY=Rows-1;TellerY>=0;TellerY--)
			{
				if(PlayField[0][TellerX][TellerY] == BlockNr)
				{
					PlayField[0][TellerX][TellerY] = 0;
					PlayField[1][TellerX][TellerY] = BlockNr;
				}
			}
	}
}

bool MakeBlockUnActive()
{
	int X,Y;
	bool FirstBlockFound,UnActive;
	FirstBlockFound = false;
	UnActive = true;
	for (X=Cols-1;X>=0;X--)
		for(Y=Rows-1;Y>=0;Y--)
			if ((PlayField[1][X][Y] > 0)&& (PlayField[0][X][Y] != 0))
			{
				UnActive = false;
				break;
			}
	if(UnActive)
		for(X=Cols-1;X >=0; X--)
			for(Y=Rows-1 ; Y >=0 ;Y--)
				if(PlayField[1][X][Y] > 0)
				{
					if (!FirstBlockFound)
					{
						Hand.SetPosition(X, Y );
						FirstBlockFound = true;
					}
					PlayField[0][X][Y] = PlayField[1][X][Y];
					PlayField[1][X][Y] = 0;
				}
	return UnActive;
}

void FlipBlock(const bool Horizontal)
{
	int BlockW,BlockH,X,Y,MinX,MinY,MaxX,MaxY;
	int Tmp[Cols][Rows];
	MinX = 100;
	MinY = 100;
	MaxX = -1;
	MaxY = -1;
	for (X = Cols-1 ; X >=0; X--)
		for(Y = Rows-1; Y>=0;Y--)
		{
			if(PlayField[1][X][Y] > 0)
			{
				if(MinX > X)
					MinX = X;
				if(MaxX < X)
					MaxX = X;
				if(MinY > Y)
					MinY = Y;
				if(MaxY < Y)
					MaxY = Y;
				Tmp[X][Y] = 0;
			}
		}

	BlockW = MaxX - MinX;
	BlockH = MaxY - MinY;
	for(X=BlockW;X>=0;X--)
		for(Y=BlockH;Y>=0;Y--)
		{
			if(Horizontal)
				Tmp[X][BlockH- Y] = PlayField[1][MinX + X][MinY + Y];
			else
				Tmp[BlockW - X][Y] = PlayField[1][MinX+X][MinY + Y];
			PlayField[1][MinX+X][MinY+Y] = 0;
		}
	for(X=BlockW;X>=0;X--)
		for(Y=BlockH;Y>=0;Y--)
			PlayField[1][MinX+X][MinY+Y] = Tmp[X][Y];
	if (GlobalSoundEnabled && SoundEnabled)
		Mix_PlayChannel(-1,Sounds[SND_FlipBlock],0);
}

void RotateBlock()
{
 	int Tmp1,Tmp2,Offset,NewBlockW,NewBlockH,BlockW,BlockH,MinX,MinY,MaxX,MaxY,X,Y,Z;
 	int Tmp[Cols][Rows];
	MinX = 100;
	MinY = 100;
	MaxX = -1;
	MaxY = -1;
	for (X = Cols-1 ; X >=0; X--)
		for(Y = Rows-1; Y>=0;Y--)
		{
			if(PlayField[1][X][Y] > 0)
			{
				if(MinX > X)
					MinX = X;
				if(MaxX < X)
					MaxX = X;
				if(MinY > Y)
					MinY = Y;
				if(MaxY < Y)
					MaxY = Y;
			}
			Tmp[X][Y] = 0;
		}

	BlockW = MaxX - MinX +1;
	BlockH = MaxY - MinY +1;
	if(BlockW == BlockH)
 	{
 		for(X=BlockW-1;X>=0;X--)
 			for(Y=BlockH-1;Y>=0;Y--)
 			{
 				Tmp[BlockW-1-Y][X] = PlayField[1][X+MinX][Y+MinY];
 				PlayField[1][X+MinX][Y+MinY] = 0;
 			}
 		for(X=BlockW-1;X>=0;X--)
 			for(Y=BlockH-1;Y>=0;Y--)
 				PlayField[1][MinX+X][MinY+Y] = Tmp[X][Y];

 	}

 	if(BlockW > BlockH)
 	{
  		NewBlockH = BlockW;
 		if((MinY + ( NewBlockH / 2) >= Rows) || (MinY - ((NewBlockH -1)/ 2) <0) || (MinX + BlockW / 2 > Cols))
 		{
 			if(GlobalSoundEnabled && SoundEnabled)
 				Mix_PlayChannel(-1,Sounds[SND_Error],0);
 			return;
 		}
 		for(X=BlockW-1;X>=0;X--)
 		{
 			for(Y=BlockH-1;Y>=0;Y--)
 			{
 				Tmp[BlockW-1-Y][X] = PlayField[1][X+MinX][Y+MinY];
 				PlayField[1][X+MinX][Y+MinY] = 0;
 			}
 		}
 		Tmp1 = (BlockW - 1) / 2;
 		Tmp2 = (NewBlockH-1)/ 2;
 		for(X=BlockW-1;X>=0;X--)
 			for(Y=NewBlockH-1;Y>=0;Y--)
 			{
 				if((MinX + X -  Tmp1 >=0) && (MinX + X - Tmp1 <= Cols-1) &&
 				   (MinY + Y -  Tmp2 >= 0) && (MinY+Y - Tmp2/ 2 <= Rows-1))
					PlayField[1][MinX+X - Tmp1][MinY+Y - Tmp2] = Tmp[X][Y];
 			}
 	}
 	if(BlockW < BlockH)
 	{
 		NewBlockW = BlockH;
 		NewBlockH = BlockW;
		if ((BlockW == 2) && (BlockH == 3))
			Offset = 1;
		else
			Offset = 0;
		if((MinX + Offset +  NewBlockW / 2 >= Cols) || (MinX + Offset - (NewBlockW-1)/2 <0) || (MinY + (BlockH) /2 > Rows))
		{
			if(GlobalSoundEnabled && SoundEnabled)
 				Mix_PlayChannel(-1,Sounds[SND_Error],0);
 			return;
		}
		for(X=BlockW-1;X>=0;X--)
			for(Y=BlockH-1;Y>=0;Y--)
			{
				Tmp[NewBlockW-1-Y][X] = PlayField[1][X+MinX][Y+MinY];
				PlayField[1][X+MinX][Y+MinY] = 0;
			}

		Tmp1 = (NewBlockW-1) / 2;
		Tmp2 = (BlockH -1) / 2;
		for(X=NewBlockW-1;X>=0;X--)
			for(Y=NewBlockH-1;Y>=0;Y--)
			{
				if((MinX+X + Offset - Tmp1  >= 0) && (MinX+X +Offset - Tmp1 <= Cols-1) &&
				   (MinY+Y + Tmp2  >=0) && (MinY+Y + Tmp2 <= Rows-1))
				   	PlayField[1][MinX+X + Offset - Tmp1][MinY + Y +  Tmp2] = Tmp[X][Y];
			}
	}
	if(GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_RotateBlock],0);
}




void DrawPlayField()
{
	int Layers,X,Y;
	SDL_Rect DstRect;
	for (Layers = 0;Layers <= 1;Layers++)
		for (X=Cols-1;X>=0;X--)
			for(Y=Rows-1;Y>=0;Y--)
			{
				if (PlayField[Layers][X][Y] > 0)
				{
					DstRect.x = XOffsetGame + X * BlockWidth;
					DstRect.y = YOffsetGame + Y * BlockHeight;
					DstRect.w = BlockWidth;
					DstRect.h = BlockHeight;
					SDL_BlitSurface(BlockImages[PlayField[Layers][X][Y] -1],NULL,Buffer,&DstRect);
				}

				if (PlayField[Layers][X][Y] < 0)
				{
					DstRect.x = XOffsetGame + X * BlockWidth;
					DstRect.y = YOffsetGame + Y * BlockHeight;
					DstRect.w = BlockWidth;
					DstRect.h = BlockHeight;
					SDL_BlitSurface(BorderImages[abs(PlayField[Layers][X][Y]) -1],NULL,Buffer,&DstRect);
				}
			}
}

void LoadLevel()
{
	int X,Y, Value;
	FILE *PFile;
	char FileName[FILENAME_MAX];
	sprintf(FileName,"./levels/level%d.lev",Level);
	PFile = fopen(FileName,"r");
	if(PFile)
	{
		printf("File load Success:%s\n",FileName);
		for (X=0;X<Cols;X++)
		{ 	for (Y=0;Y <Rows;Y++)
	 		{
	 			fscanf(PFile,"%d,",&Value);
	 			PlayField[0][X][Y] = Value;
	 			PlayField[1][X][Y] = 0;
	 		}
		}
		fclose(PFile);
	}
	else
	{
		printf("File load Failed:%s\n",FileName);
	}
}

CHand::CHand()
{
	Image = IMG_Load("./graphics/hand.png");
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
	Hidden = false;
	X = 0;
	Y = 0;
}

void CHand::Draw()
{
	SDL_Rect DstRect;
	if (!Hidden)
	{
		DstRect.x = MinPlayAreaX + X * BlockWidth;
		DstRect.y = MinPlayAreaY + Y * BlockHeight;
		DstRect.w = Image->w;
		DstRect.h = Image->h;
		SDL_BlitSurface(Image,NULL,Buffer,&DstRect);
	}
}

void CHand::Move()
{
	if (!Hidden)
	{
		if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_UP))
		{
			if (Y - 1 >= 0)
				Y = Y - 1;
		}
		else if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_DOWN))
		{
			if (Y + 1 < Rows)
				Y = Y + 1;
		}
		else if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_LEFT))
		{
			if (X - 1 >= 0)
				X = X - 1;
		}
		else if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_RIGHT))
		{
			if (X + 1 < Cols)
				X = X + 1;
		}
		else if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_UPRIGHT))
		{
			if (Y - 1 >= 0)
				Y = Y - 1;
			if (X + 1 < Cols)
				X = X + 1;
		}
		else if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_UPLEFT))
		{
			if (Y - 1 >= 0)
				Y = Y - 1;
			if (X - 1 >= 0)
				X = X - 1;
		}
		else if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_DOWNRIGHT))
		{
			if (Y + 1 < Rows)
				Y = Y + 1;
			if (X + 1 <= Cols)
				X = X + 1;
		}
		else if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_DOWNLEFT))
		{
			if (Y + 1 < Rows)
				Y = Y + 1;
			if (X - 1 >= 0)
				X = X - 1;
		}
	}
}

void CHand::SetPosition(const int XIn,const int YIn)
{
	X = XIn;
	Y = YIn;
}

int CHand::GetPlayFieldX()
{
	return X;
}

int CHand::GetPlayFieldY()
{
	return Y;
}

void CHand::Hide()
{
	Hidden = true;
}

void CHand::Show()
{
	Hidden = false;
}

CHand::~CHand()
{
	SDL_FreeSurface(Image);
}

bool IsStageClear()
{
	int X,Y,TellerX,TellerY;
	bool StartBorderFound,VertStartBorderFound,VertEndBorderFound,Temp;
	Temp = true;
	if (Level == 17)
	{
		for(Y=6;Y<=8;Y++)
			for(X=0;X<Cols;X++)
				Temp = Temp && (PlayField[0][X][Y] > 0);
	}
	else
	if (Level == 32)
	{
		for (Y=0;Y<=14;Y++)
			for (X=7;X<=9;X++)
				Temp = Temp && (PlayField[0][X][Y] > 0);
		for (Y=6;Y<=8;Y++)
			for (X = 10;X<=14;X++)
				Temp = Temp && (PlayField[0][X][Y] > 0);

	}
	else
	{
		StartBorderFound = false;
		for (Y=0;Y<Rows;Y++)
			for (X=0;X<Cols;X++)
			{
				if (!StartBorderFound)
				{
					if(X < Cols-1)
						if((PlayField[0][X][Y] < 0) && (PlayField[0][X+1][Y] >=0))
							for(TellerX = X+1;TellerX<Cols;TellerX++)
								if(PlayField[0][TellerX][Y] < 0)
									StartBorderFound = true;
				}
				else
				{
					if(PlayField[0][X][Y] < 0)
					{
						StartBorderFound = false;
						if( X < Cols-1)
							if((PlayField[0][X][Y] < 0) && (PlayField[0][X+1][Y] >= 0))
								for(TellerX = X+1;TellerX < Cols;TellerX++)
									if(PlayField[0][TellerX][Y] < 0)
										StartBorderFound=true;
						continue;
					}

					if (StartBorderFound)
					{
						VertEndBorderFound = false;
						VertStartBorderFound = false;
						if (Y < Rows -1)
							for (TellerY=Y+1;TellerY<Rows;TellerY++)
								if(PlayField[0][X][TellerY] < 0)
									VertEndBorderFound = true;
						if (Y > 0)
							for(TellerY = Y -1; TellerY >=0;TellerY--)
								if(PlayField[0][X][TellerY] < 0)
									VertStartBorderFound = true;
						if(VertStartBorderFound && VertEndBorderFound)
							Temp = Temp && (PlayField[0][X][Y] > 0);
					}

				}
			}


	}
	return Temp;
}


void DrawPanel()
{
	SDL_Surface* Text;
	SDL_Rect DstRect;
	char ChrText[100];
	sprintf(ChrText,"Room Number: %d",Level);
	Text = TTF_RenderText_Solid(font,ChrText,Color);
	DstRect.x = 30;
	DstRect.y = 0;
	DstRect.w = Text->w;
	DstRect.h = Text->h;
	SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
	SDL_FreeSurface(Text);
	sprintf(ChrText,"Time: %02d:%02d:%02d",(EndTime - StartTime) / 3600, ((EndTime - StartTime)%3600)/60,(EndTime-StartTime)%60);
	Text = TTF_RenderText_Solid(font,ChrText,Color);
	DstRect.x = 175;
	DstRect.y = 0;
	DstRect.w = Text->w;
	DstRect.h = Text->h;
	SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
	SDL_FreeSurface(Text);
}
void Game()
{
	printf("Game Started\n");
	int PlayFieldX,PlayFieldY,X,Y,Layers,BlockNr,framecount,frames;
	printf("Game Started 2\n");
	SDL_Rect DstRect = {0,200,0,0};
	printf("Game Started 3\n");
	SDL_Surface *Background,*Text;
	printf("Game Started 4\n");
	SDL_Event Event;
	printf("Game Started 5\n");
	Background = IMG_Load("./graphics/roombackground.png");
	printf("Game Started 6\n");
	LoadLevel();
	Hand.SetPosition(10, 8);
	Hand.Show();
	srand ( time(NULL) );
	Uint32 OldTime,NewTime;
	BlockActive = false;
	OldTime = 0;
	char fps[20];
	StartTime = time(NULL);
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	switch(rand()%2)
	{
		case 0 : if(GlobalSoundEnabled && MusicEnabled)
				 	Mix_PlayMusic(Music[MUS_Game1],-1);
				 break;
		case 1 : if(GlobalSoundEnabled && MusicEnabled)
				 	Mix_PlayMusic(Music[MUS_Game2],-1);
				 break;

	}

	SDL_PollEvent(&Event);
	while (GameState == GSGame)
	{
		while(SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					case GP2X_BUTTON_SELECT :
						GameState = GSTitleScreen;
						Mix_HaltMusic();
						break;
				/*	case GP2X_BUTTON_L:
						DebugPrint();
						break;
									*/
					case GP2X_BUTTON_CLICK:
					case GP2X_BUTTON_A :
						if(!BlockActive)
						{
							PlayFieldX = Hand.GetPlayFieldX();
							PlayFieldY = Hand.GetPlayFieldY();
							BlockNr = PlayField[0][PlayFieldX][PlayFieldY];
							if (BlockNr > 0)
							{
								if(GlobalSoundEnabled && SoundEnabled)
									Mix_PlayChannel(-1,Sounds[SND_PickupBlock],0);
								BlockActive = true;
								Hand.Hide();
								MakeBlockActive(PlayFieldX,PlayFieldY,BlockNr);
							}
						}
						else
						{
							if(MakeBlockUnActive())
							{
								if(GlobalSoundEnabled && SoundEnabled)
									Mix_PlayChannel(-1,Sounds[SND_DropBlock],0);
								BlockActive = false;
								if(IsStageClear())
								{
									GameState = GSStageClear;
									if(GlobalSoundEnabled && SoundEnabled)
										Mix_PlayChannel(-1,Sounds[SND_StageEnd],0);
								}
								else
									Hand.Show();
							}
							else
								if(GlobalSoundEnabled && SoundEnabled)
									Mix_PlayChannel(-1,Sounds[SND_Error],0);
						}
						break;
					case GP2X_BUTTON_Y:
						if(BlockActive)
							FlipBlock(true);
						break;
					case GP2X_BUTTON_X:
						if(BlockActive)
							FlipBlock(false);
						break;
					case GP2X_BUTTON_B:
						if(BlockActive)
							RotateBlock();
						break;
					case GP2X_BUTTON_START:
						LoadLevel();
						BlockActive = false;
						Hand.SetPosition(10,8);
						Hand.Show();
						break;
				/*	case GP2X_BUTTON_R:
						Level++;
						break; */
					}
			}
		}
		if (BlockActive)
		{
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_RIGHT))
			{
				MoveBlockRight();
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_LEFT))
			{
				MoveBlockLeft();
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_UP))
			{
				MoveBlockUp();
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_DOWN))
			{
				MoveBlockDown();
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_UPRIGHT))
			{
				MoveBlockRight();
				MoveBlockUp();
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_UPLEFT))
			{
				MoveBlockLeft();
				MoveBlockUp();
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_DOWNRIGHT))
			{
				MoveBlockRight();
				MoveBlockDown();
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_DOWNLEFT))
			{
				MoveBlockLeft();
				MoveBlockDown();
			}
		}
		EndTime = time(NULL);
		sprintf(fps,"fps: %d",1000 /(SDL_GetTicks() - OldTime));
		OldTime=SDL_GetTicks();
		Text = TTF_RenderText_Solid(font,fps,Color);
		SDL_BlitSurface(Background,NULL,Buffer,NULL);
		DrawPanel();
		//SDL_BlitSurface(Text,NULL,Screen,&DstRect);
		SDL_FreeSurface(Text);
		DrawPlayField();
		Hand.Move();
		Hand.Draw();
		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
	}
	SDL_FreeSurface(Background);
	printf("Game Ended\n");
}

//Stage Clear stuff ----------------------------------------------------
void StageClear()
{
    SDL_Event Event;
 	SDL_Rect DstRect;
 	bool KaderVisible;
 	KaderVisible = true;
 	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
 	SDL_PollEvent(&Event);
 	while (GameState == GSStageClear)
 	{
 		 while(SDL_PollEvent(&Event))
		 {
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					case GP2X_BUTTON_SELECT :
						Mix_HaltChannel(0);
						Mix_HaltChannel(1);
						Mix_HaltMusic();
						GameState = GSTitleScreen;
						break;
					case GP2X_BUTTON_A :
					case GP2X_BUTTON_START:
					case GP2X_BUTTON_X:
						Mix_HaltChannel(0);
						Mix_HaltChannel(1);
						Mix_HaltMusic();
						GameState = GSOldManSpeaking;
						break;
					case GP2X_BUTTON_Y:
					case GP2X_BUTTON_B:
					case GP2X_BUTTON_L:
					case GP2X_BUTTON_R:
						KaderVisible = !KaderVisible;
						break;
				}
			}
		}
 		DstRect.x = XOffsetGame-4;
  		DstRect.y = YOffsetGame + 75;
  		DstRect.w = StageClearKader->w;
  		DstRect.h = StageClearKader->h;
  		SDL_BlitSurface(RoomBackground,NULL,Buffer,NULL);
  		DrawPanel();
  		DrawPlayField();
  		if (KaderVisible)
  		  SDL_BlitSurface(StageClearKader,NULL,Buffer,&DstRect);
  		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
	}
}


CPlayer::CPlayer(const int Xin,const int Yin,const int MinXin, const int MaxXin)
{
 	Image = IMG_Load("./graphics/ryf-player.png");
 	Shadow = IMG_Load("./graphics/ryf-shadow.png");
 	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
 	SDL_SetColorKey(Shadow,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Shadow->format,255,0,255));
 	SDL_SetAlpha(Shadow,SDL_SRCALPHA,128);
 	Y = Yin;
	X = Xin;
 	Width = 23;
 	Height = 28;
 	AnimCounter = 0;
 	Delay = 0;
 	AnimPhase = 0;
 	MaxX = MaxXin;
 	MinX = MinXin;
 	State = Waiting;
}

void CPlayer::Draw()
{
	SDL_Rect DstRect,SrcRect;
	DstRect.x = X + 1;
 	DstRect.y = Y + 26;
 	DstRect.w = Shadow->w;
 	DstRect.h = Shadow->h;
 	SDL_BlitSurface(Shadow,NULL,Buffer,&DstRect);

 	DstRect.x = X;
 	DstRect.y = Y;
 	DstRect.w = Width;
 	DstRect.h = Height;
 	SrcRect.x = AnimPhase * Width;
 	SrcRect.y = 0;
 	SrcRect.w = Width;
 	SrcRect.h = Height;
	SDL_BlitSurface(Image,&SrcRect,Buffer,&DstRect);
}

int CPlayer::GetX()
{
	return X;
}

int CPlayer::GetY()
{
	return Y;
}

int CPlayer::GetWidth()
{
	return Width;
}

CPlayer::~CPlayer()
{
 	SDL_FreeSurface(Shadow);
 	SDL_FreeSurface(Image);
}

void CPlayer::Move()
{

	if ((State == Walking) || (State==Waiting))
	{
   		if(SDL_JoystickGetButton(Joystick,GP2X_BUTTON_LEFT))
		{
			if(X - 3 >= MinX)
			{
				X = X -3;
				AnimPhase = AnimLeft[AnimCounter];
				State = Walking;
			}
			else
				State = Waiting;
		}
 		else
			if(SDL_JoystickGetButton(Joystick,GP2X_BUTTON_RIGHT))
			{
				if(X + 3 <= MaxX)
				{
					X = X +3;
					AnimPhase = AnimRight[AnimCounter];
					State = Walking;
				}
				else
					State = Waiting;
			}
			else
				State = Waiting;


	}
	if (State == Walking)
	{
     	AnimCounter++;
     	if (AnimCounter > 5)
     		AnimCounter = 0;
    }
  	if (State == EnterBuilding)
  	{
  		Delay++;
  		if(Delay==3)
  		{
  			AnimCounter++;
  			Y = Y - 1;
  			Delay = 0;
  		}
  		if (AnimCounter > 3)
  		{
  			AnimCounter = 0;
			if(Y <=160)
			{
			  State = EnteredBuilding;
			  printf("Entered building");
			 }
		}
 		AnimPhase = AnimEnterBuilding[AnimCounter];
	}
	if (State == Waiting)
	{
		if ((AnimPhase != 0) && (AnimPhase !=13))
			if (AnimPhase==14)
				AnimPhase = 0;
			else
				if (AnimPhase < 7)
					AnimPhase = 0;
				else
					AnimPhase=13;
	}
 	if (State ==LookingUp)
 		AnimPhase = 14;

}

CSpaceShip::CSpaceShip()
{
	Image = IMG_Load("./graphics/spaceship.png");
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
	X=-32;
	Y= 28;
}

void CSpaceShip::Draw()
{
	SDL_Rect DstRect;
	DstRect.x = X;
	DstRect.y = Y;
	DstRect.w = Image->w;
	DstRect.h = Image->h;
	SDL_BlitSurface(Image,NULL,Buffer,&DstRect);
}

void CSpaceShip::Move()
{
	if (X < 352)
		X = X + 3;
}

int CSpaceShip::GetX()
{
	return X;
}

CSpaceShip::~CSpaceShip()
{
	SDL_FreeSurface(Image);
}

CCloud::CCloud(const int XIn,const int YIn,float XiIn,CloudStyles Style)
{
	if (Style == Big)
	{
		Image = IMG_Load("./graphics/ryf-cloud.png");
		Width = 60;
		Height = 28;
	}
	else
	{
		Image = IMG_Load("./graphics/ryf-smallcloud.png");
		Width=30;
		Height=14;
	 }
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
	X = XIn;
	Y = YIn;
	Xi = XiIn;
}

void CCloud::Draw()
{
	SDL_Rect SrcRect,DstRect;
	if ((X < 0) && (X >= -Width))
	{
		SrcRect.x = abs((int)X);
		SrcRect.y = 0;
		SrcRect.w = abs((int)X) + Width;
		SrcRect.h = Height;
		DstRect.x = (int)X;
		DstRect.y = Y;
		DstRect.w = (int)X + Width;
		DstRect.h = Height;
		SDL_BlitSurface(Image,&SrcRect,Buffer,&DstRect);
	}
	else
		if((X >=0) && (X <= 319 - Width))
		{
			SrcRect.x = 0;
			SrcRect.y = 0;
			SrcRect.w = Width;
			SrcRect.h = Height;
			DstRect.x = (int)X;
			DstRect.y = Y;
			DstRect.w = Width;
			DstRect.h = Height;
			SDL_BlitSurface(Image,&SrcRect,Buffer,&DstRect);
		}
		else
			if((X > 319 - Width) && (X < 320))
			{
				SrcRect.x = 0;
				SrcRect.y =0;
				SrcRect.w = 319 - (int)X;
				SrcRect.h = Height;
				DstRect.x = (int)X;
				DstRect.y = Y;
				DstRect.w = 319-(int)X;
				DstRect.h = Height;
				SDL_BlitSurface(Image,&SrcRect,Buffer,&DstRect);
			}
}

void CCloud::Move()
{
	if ((X > -Width) && (X < 320))
		X = X + Xi;
	else
		if (X <= -Width)
			X = 319;
		else
			X = -Width + 1;
}


CCloud::~CCloud()
{
	SDL_FreeSurface(Image);
}

CFairy::CFairy(const int XIn, const int YIn,const int AnimDelayIn)
{
	Image = IMG_Load("./graphics/ryf-fairy.png");
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
	X = XIn;
	Y = YIn;
	Width = 18;
	Height = 22;
	AnimPhase = 0;
	Delay = 0;
	Hidden = false;
	AnimDelay = AnimDelayIn;
}

void CFairy::Draw()
{
	SDL_Rect SrcRect,DstRect;
	if (!Hidden)
	{
		SrcRect.x = AnimPhase * Width;
		SrcRect.y = 0;
		SrcRect.w = Width;
		SrcRect.h = Height;
		DstRect.x = X;
		DstRect.y = Y;
		DstRect.w = Width;
		DstRect.h = Height;
		SDL_BlitSurface(Image,&SrcRect,Buffer,&DstRect);
		Delay++;
		if (Delay >= AnimDelay)
		{
			AnimPhase++;
			Delay = 0;
			if (AnimPhase == 2)
				AnimPhase = 0;
		}
	}
}

int CFairy::GetX()
{
	return X;
}

CFairy::~CFairy()
{
	SDL_FreeSurface(Image);
}

CStageBlock::CStageBlock()
{
	Image = NULL;
}

void CStageBlock::Load(const int BlockNr)
{
	char ChrFileName[FILENAME_MAX];
	sprintf(ChrFileName,"./graphics/stageblock%d.png",BlockNr);
	Image = IMG_Load(ChrFileName);
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
	Hidden = false;
	Yi = 3;
	X = 160 - Image->w / 2;
	Y = -Image->h;
	Width = Image->w;
	Height = Image->h;
}

void CStageBlock::Draw()
{
	SDL_Rect DstRect;
	if (!Hidden)
	{
		DstRect.x = X;
		DstRect.y = Y;
		DstRect.w = Width;
		DstRect.h = Height;
		SDL_BlitSurface(Image,NULL,Buffer,&DstRect);
	}
}

void CStageBlock::Move()
{
	if (!Hidden)
		Y = Y + Yi;
}

int CStageBlock::GetY()
{
	return Y;
}

int CStageBlock::GetHeight()
{
	return Height;
}

CStageBlock::~CStageBlock()
{
	if (Image != NULL)
	{
		SDL_FreeSurface(Image);
	}
}

void NextStageLevel1to35()
{
	char ChrLevel[5];

	CStageBlock StageBlock;
	CPlayer Player(30,167,30,268);
	CCloud Cloud1(259,15,-0.40,Big);
	CCloud Cloud2(185,45,-0.25,Small);
	CCloud Cloud3(134,7,-0.40,Big);
	CCloud Cloud4(50,50,-0.25,Small);
	CSpaceShip SpaceShip;
	SDL_Color Color1 = {25,50,50,0};
	SDL_Color Color2 = {0,0,0,0};
	CFairy Fairy(172,160,3);
	SDL_Event Event;
	bool BridgeShown,BridgeDrawing;
	int BridgeDrawnWidth;
	SDL_Surface *BackGround,*Text,*PrevLevel,*NextLevel;
	SDL_Rect PrevLevelDstRect,NextLevelDstRect,BridgeSrcRect,BridgeDstRect,TextDstRect;
	BridgeShown = false;
	BridgeDrawing = false;
	BridgeDrawnWidth = 0;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	BackGround = IMG_Load("./graphics/betweenstage.png");
	if (GlobalSoundEnabled && MusicEnabled)
		Mix_PlayMusic(Music[MUS_Stage],-1);
	switch (Level)
	{
		case 1:
			StageBlock.Load(1);
			break;
  		case 2:
			StageBlock.Load(2);
  			break;
  		case 3:
			StageBlock.Load(3);
  			break;
  		case 4:
			StageBlock.Load(4);
  			break;
  		case 5:
			StageBlock.Load(5);
  			break;
  		case 6:
			StageBlock.Load(6);
  			break;
  		case 8:
  			StageBlock.Load(7);
  			break;
  		case 9:
  			StageBlock.Load(8);
  			break;
  		case 11:

  			StageBlock.Load(9);
  			break;
  		case 35:
  			StageBlock.Load(10);
  			break;
  		default: Fairy.Hidden = true;
	}
	Text = TTF_RenderText_Solid(font,LevelPasswords[Level],Color1);
	TextDstRect.x = 160 - Text->w / 2;
	TextDstRect.y = 25;
	TextDstRect.w = Text->w;
	TextDstRect.h = Text->h;
	sprintf(ChrLevel,"%02d",Level+1);
	NextLevel = TTF_RenderText_Solid(font,ChrLevel,Color2);
	NextLevelDstRect.x = 270;
	NextLevelDstRect.y = 102;
	NextLevelDstRect.w = NextLevel->w;
	NextLevelDstRect.h = NextLevel->h;
	sprintf(ChrLevel,"%02d",Level);
	PrevLevel = TTF_RenderText_Solid(font,ChrLevel,Color2);
	PrevLevelDstRect.x = 31;
	PrevLevelDstRect.y = 102;
	PrevLevelDstRect.w = PrevLevel->w;
	PrevLevelDstRect.h = PrevLevel->h;
	SDL_PollEvent(&Event);
	while (GameState == GSNextStage)
	{
		while(SDL_PollEvent(&Event))
		 {
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					case GP2X_BUTTON_SELECT :
						GameState = GSTitleScreen;
						break;
					case GP2X_BUTTON_CLICK:
					case GP2X_BUTTON_A :
					case GP2X_BUTTON_START:
					case GP2X_BUTTON_X:
						if(Player.GetX() > 265)
						{
							Level++;
							Player.State = EnterBuilding;
						}
						if(Player.GetX() < 32)
							Player.State = EnterBuilding;
						break;
				}
			}
		}
		sprintf(chrdebug,"Player x:%d\n",Player.GetX());
		//printf("%s",chrdebug);
		SDL_BlitSurface(BackGround,NULL,Buffer,NULL);
		Cloud1.Move();
  		Cloud1.Draw();
  		Cloud2.Move();
  		Cloud2.Draw();
  		Cloud3.Move();
  		Cloud3.Draw();
  		Cloud4.Move();
  		Cloud4.Draw();
		//printf("BridgeStart\n");
		if (!BridgeShown)
		{
			if(!BridgeDrawing)
			{
				if(Player.GetX() > 92)
				{
					BridgeDrawing = true;
					if (GlobalSoundEnabled && SoundEnabled)
						Mix_PlayChannel(-1,Sounds[SND_Bridge],0);
				}
			}
			else
			{
				BridgeSrcRect.x = 0;
				BridgeSrcRect.y = 0;
				BridgeSrcRect.w = BridgeDrawnWidth;
				BridgeSrcRect.h = Bridge->h;
				BridgeDstRect.x = 111;
				BridgeDstRect.y = 188;
				BridgeDstRect.w = BridgeDrawnWidth;
				BridgeDstRect.h = Bridge->h;
				BridgeDrawnWidth = BridgeDrawnWidth + 5;
				if (BridgeDrawnWidth >= Bridge->w)
				{
					BridgeDrawing = false;
					BridgeShown = true;
					BridgeSrcRect.w = Bridge->w;
					BridgeDstRect.w = Bridge->w;
				}
				SDL_BlitSurface(Bridge,&BridgeSrcRect,Buffer,&BridgeDstRect);
			}
		}
		else
			SDL_BlitSurface(Bridge,NULL,Buffer,&BridgeDstRect);
		Player.Move();
		Player.Draw();
		//printf("Switch level Start\n");
		switch(Level)
		{
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 8:
			case 9:
			case 11:
			case 35:
				Fairy.Draw();
			//	printf("in switch level b\n");
				if((Player.State != LookingUp) && (! Fairy.Hidden) && (Player.GetX() + Player.GetWidth() >= Fairy.GetX()))
				{
					if (GlobalSoundEnabled && SoundEnabled)
						Mix_PlayChannel(-1,Sounds[SND_Elf],0);
					printf("Sound Played\n");
					Fairy.Hidden = true;
					Player.State = LookingUp;
				}
				if (Player.State == LookingUp)
				{
					StageBlock.Draw();
					StageBlock.Move();
					if( StageBlock.GetY() + StageBlock.GetHeight() >= Player.GetY())
					{
						Player.State = Waiting;
						StageBlock.Hidden = true;
					}
				}
				break;
		}
		SpaceShip.Move();
		if (SpaceShip.GetX() > 160-Text->w /2)
			SDL_BlitSurface(Text,NULL,Buffer,&TextDstRect);
		SpaceShip.Draw();
		SDL_BlitSurface(PrevLevel,NULL,Buffer,&PrevLevelDstRect);
		SDL_BlitSurface(NextLevel,NULL,Buffer,&NextLevelDstRect);
		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
		if (Player.State == EnteredBuilding)
			GameState = GSGame;
		//printf("While End\n");
	}
	Mix_HaltMusic();
	SDL_FreeSurface(PrevLevel);
 	SDL_FreeSurface(NextLevel);
	SDL_FreeSurface(Text);
	SDL_FreeSurface(BackGround);
	printf("Next Stage level 1 - 35 Ended\n");
}


void NextStageLevel0()
{
	CPlayer Player(225,167,225,268);
	CCloud Cloud1(259,15,-0.40,Big);
	CCloud Cloud2(185,45,-0.25,Small);
	CCloud Cloud3(134,7,-0.40,Big);
	CCloud Cloud4(50,50,-0.25,Small);
	SDL_Color Color1 ={0,0,0,0};
	SDL_Event Event;
	SDL_Surface *BackGround,*Text;
	SDL_Rect DstRect;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	BackGround = IMG_Load("./graphics/betweenstagelevel1.png");
	char ChrLevel[5];
	if (GlobalSoundEnabled && MusicEnabled)
		Mix_PlayMusic(Music[MUS_Stage],-1);
	sprintf(ChrLevel,"%02d",Level+1);
	Text = TTF_RenderText_Solid(font,ChrLevel,Color1);
	DstRect.x = 270;
	DstRect.y = 105;
	DstRect.w = Text->w;
	DstRect.h = Text->h;
	SDL_PollEvent(&Event);
	while (GameState == GSNextStage)
	{
		while(SDL_PollEvent(&Event))
		 {
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					case GP2X_BUTTON_SELECT :
						Mix_HaltMusic();
						GameState = GSTitleScreen;
						break;
					case GP2X_BUTTON_CLICK:
					case GP2X_BUTTON_A :
					case GP2X_BUTTON_START:
					case GP2X_BUTTON_X:
						if(Player.GetX() > 265)
						{
							Level++;
							Player.State = EnterBuilding;
						}
						break;
				}
			}
		}
		SDL_BlitSurface(BackGround,NULL,Buffer,NULL);
		SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
		Cloud1.Move();
		Cloud1.Draw();
		Cloud2.Move();
		Cloud2.Draw();
		Cloud3.Move();
		Cloud3.Draw();
		Cloud4.Move();
		Cloud4.Draw();
		Player.Move();
		Player.Draw();
		if (Player.State == EnteredBuilding)
			GameState = GSGame;
		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
	}
	Mix_HaltMusic();
	SDL_FreeSurface(Text);
	SDL_FreeSurface(BackGround);
}

COptionsSelector::COptionsSelector()
{
	Image = IMG_Load("./graphics/optionsselect.png");
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
	Selection = 1;
}

void COptionsSelector::Draw()
{
	SDL_Rect DstRect;
	switch(Selection)
	{
		case 1 :
			DstRect.y = 42;
			break;
		case 2 :
			DstRect.y = 62;
			break;
		case 3 :
			DstRect.y = 82;
			break;
	}
	DstRect.x = 100;
	DstRect.w = Image->w;
	DstRect.h = Image->h;
	SDL_BlitSurface(Image,NULL,Buffer,&DstRect);
}

int COptionsSelector::GetSelection()
{
	return Selection;
}

void COptionsSelector::MoveDown()
{
	if (Selection < 2)
	{
		Selection++;
		if(GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}
}

void COptionsSelector::MoveUp()
{
	if (Selection > 1)
	{
		Selection--;
		if(GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}
}

COptionsSelector::~COptionsSelector()
{
	SDL_FreeSurface(Image);
}

void Options()
{
	SDL_Surface *Background;
	SDL_Event Event;
	SDL_Color Color1 = {0,0,0,0};
	char Msg[20];
	COptionsSelector Selector;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	Background = IMG_Load("./graphics/paper.png");
	SDL_PollEvent(&Event);
	while (GameState == GSOptions)
	{
		while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					case GP2X_BUTTON_SELECT:
					case GP2X_BUTTON_B:
					case GP2X_BUTTON_Y:
						GameState = GSTitleScreen;
						break;
					case GP2X_BUTTON_CLICK:
					case GP2X_BUTTON_A:
					case GP2X_BUTTON_X:
					case GP2X_BUTTON_START:
						if(Selector.GetSelection() == 1)
							SoundEnabled = ! SoundEnabled;
						if(Selector.GetSelection() == 2)
						{
							MusicEnabled = ! MusicEnabled;
							if (!MusicEnabled)
								Mix_HaltMusic();
							else
								Mix_PlayMusic(Music[MUS_Title],-1);
						}
						if(GlobalSoundEnabled && SoundEnabled)
							Mix_PlayChannel(-1,Sounds[SND_Select],0);
						break;
					case GP2X_BUTTON_UP:
						Selector.MoveUp();
						break;
					case GP2X_BUTTON_DOWN:
						Selector.MoveDown();
						break;
				}

			}
		}
		SDL_BlitSurface(Background,NULL,Buffer,NULL);
		TTF_SetFontStyle(font,TTF_STYLE_UNDERLINE);
		WriteText(Buffer,font,"OPTIONS",7,Buffer->w / 2 - 25,20,0,Color1);
		TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
		if(SoundEnabled)
			strcpy(Msg,"SOUND: ON");
		else
			strcpy(Msg,"SOUND: OFF");
		WriteText(Buffer,font,Msg,strlen(Msg),Buffer->w / 2 -35,40,0,Color1);
		if (MusicEnabled)
			strcpy(Msg,"MUSIC: ON");
		else
			strcpy(Msg,"MUSIC: OFF");
		WriteText(Buffer,font,Msg,strlen(Msg),Buffer->w / 2 -35,60,0,Color1);
		sprintf(Msg,"VOLUME: %d%",Volume * 100/ 128);
		WriteText(Buffer,font,Msg,strlen(Msg),Buffer->w / 2 - 35,80,0,Color1);
		Selector.Draw();
		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
	}
	SDL_FreeSurface(Background);
	SaveSettings();

}

CStageSelectSelector::CStageSelectSelector()
{
	Image = IMG_Load("./graphics/select.png");
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
	X = 0;
	Y = 0;
}

void CStageSelectSelector::Draw()
{
	SDL_Rect DstRect;
	DstRect.x = XOffsetStageSelect + X * 82 + 52;
	DstRect.y = YOffsetStageSelect + Y * 14+2;
	DstRect.w = Image->w;
	DstRect.h = Image->h;
	SDL_BlitSurface(Image,NULL,Buffer,&DstRect);
}

void CStageSelectSelector::MoveDown()
{
	if (Y < 11)
	{
		Y++;
		if (GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}
}

void CStageSelectSelector::MoveLeft()
{
	if (X > 0)
	{
		X--;
		if (GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}
}

void CStageSelectSelector::MoveRight()
{
	if (X < 2)
	{
		X++;
		if (GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}
}

void CStageSelectSelector::MoveUp()
{
	if ( Y > 0)
	{
		Y--;
		if (GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}

}

int CStageSelectSelector::GetSelection()
{
	return  X + Y * 3;
}

CStageSelectSelector::~CStageSelectSelector()
{
	SDL_FreeSurface(Image);
}

void StageSelect()
{
	SDL_Event Event;
	SDL_Surface *Background,*Text;
	int Teller;
	SDL_Color Color1 = {0,0,0,0};
	SDL_Rect DstRect;
	char ChrRoom[10];
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	CStageSelectSelector Selector;
	Background = IMG_Load("./graphics/paper.png");
	SDL_PollEvent(&Event);
	while (GameState == GSStageSelect)
	{
		while(SDL_PollEvent(&Event))
		{
			if( Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					case GP2X_BUTTON_SELECT:
					case GP2X_BUTTON_Y:
					case GP2X_BUTTON_B:
						GameState = GSTitleScreen;
						break;
					case GP2X_BUTTON_CLICK:
					case GP2X_BUTTON_A:
					case GP2X_BUTTON_X:
					case GP2X_BUTTON_START:
						if (GlobalSoundEnabled && SoundEnabled)
							Mix_PlayChannel(-1,Sounds[SND_Select],0);
						Level = Selector.GetSelection();
						GameState = GSOldManSpeaking;
						Mix_HaltMusic();
						break;
					case GP2X_BUTTON_UP:
						Selector.MoveUp();
						break;
					case GP2X_BUTTON_DOWN:
						Selector.MoveDown();
						break;
					case GP2X_BUTTON_LEFT:
						Selector.MoveLeft();
						break;
					case GP2X_BUTTON_RIGHT:
						Selector.MoveRight();
						break;
				/*	case GP2X_BUTTON_UPLEFT:
						Selector.MoveUp();
						Selector.MoveLeft();
						break;
					case GP2X_BUTTON_UPRIGHT:
						Selector.MoveUp();
						Selector.MoveRight();
						break;
					case GP2X_BUTTON_DOWNLEFT:
						Selector.MoveDown();
						Selector.MoveLeft();
						break;
					case GP2X_BUTTON_DOWNRIGHT:
						Selector.MoveDown();
						Selector.MoveRight();
						break;
						*/
				}
			}
		}
		SDL_BlitSurface(Background,NULL,Buffer,NULL);
		Text = TTF_RenderText_Solid(font,"ROOM SELECT",Color1);
		DstRect.x = Buffer->w / 2 - Text->w / 2;
		DstRect.y = 20;
		DstRect.w = Text->w;
		DstRect.h = Text->h;
		SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
		SDL_FreeSurface(Text);
		for (Teller=0;Teller< 36;Teller++)
		{
			sprintf(ChrRoom,"Room %d",Teller+1);
			Text = TTF_RenderText_Solid(font,ChrRoom,Color1);
			DstRect.x = XOffsetStageSelect + (Teller % 3) * 82;
			DstRect.y = YOffsetStageSelect + (Teller / 3) * 14;
			DstRect.w = Text->w;
			DstRect.h = Text->h;
			SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
			SDL_FreeSurface(Text);
		}
		Selector.Draw();
		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
	}
	SDL_FreeSurface(Background);

}

void OldManSpeaking()
{
	const char* const Tekst[5] = {"Welcome to the world of\npuzzle land!\nIn level one we'll start with\nsimple three-block\ncombinations ...$When you've mastered level\none, we'll add another block.\nHang in there!",
								  "Congratulations you made it!\nNow we'll boggle your mind,\nthere are 2339 diffrent\nconfigurations in this stage!$Don't give up!\nRelax!",
								  "Hah! This next stage will\nsurprise you!\nYou now have 12 diffrent\nshapes to work with ...$Show me how far you can\ngo dude!",
								  "Well, well, well.\nYou seem to have made it all\nthis way! As a token of our\nappreciation for your skill,...$Here's another square block!\nTake the square block, and try\nit for the last level!$It looks simple, but there are\n16146 configurations!\n\nHey dude, you're on it again.",
          						  "Congratulations you made it!\nYou've earned the password\nfor the level select!\nthe password is 'davy'"};
    SDL_Event Event;
    SDL_Color Color1 = {0,0,0,0};
    int Lines,Chars,TextDelay,NrOfChars,PageNr,Teller,TekstNr;
    char List[100][255];
	CFairy Fairy(240,175,6);
	SDL_Surface* BackGround;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	SDL_PollEvent(&Event);
	switch (Level)
	{
		case 0:
		case 11:
		case 12:
		case 35:
		case 36:
			if (GlobalSoundEnabled && MusicEnabled)
				Mix_PlayMusic(Music[MUS_Oldman],-1);
			TextDelay = 0;
			NrOfChars = 0;
			Lines = 0;
			Chars =0;
			PageNr = 0;
			switch (Level)
			{
				case 0:
					TekstNr = 0;
					break;
				case 11:
					TekstNr = 1;
					break;
				case 12:
					TekstNr = 2;
					break;
				case 35:
					TekstNr = 3;
					break;
				case 36:
					TekstNr = 4;
					break;
				default: TekstNr = 0;
			}
			BackGround = IMG_Load("./graphics/oldman.png");
			for(Teller=0;Teller<strlen(Tekst[TekstNr]);Teller++)
			{
				if (Tekst[TekstNr][Teller] == '$')
				{
					List[Lines][Chars] = '\0';
					Lines++;
					Chars=0;
				}
				else
				{
					List[Lines][Chars] = Tekst[TekstNr][Teller];
					Chars++;
				}
			}
			List[Lines][Chars] = '\0';
			while (GameState == GSOldManSpeaking)
			{
				while(SDL_PollEvent(&Event))
				{
					if (Event.type == SDL_JOYBUTTONDOWN)
					{
						switch (Event.jbutton.button)
						{
							case GP2X_BUTTON_VOLUP:
								IncVolume();
								break;
							case GP2X_BUTTON_VOLDOWN:
								DecVolume();
								break;
						/*	case GP2X_BUTTON_L:
								int x;
								for(x = 0; x<= Lines;x++)
								{
									printf("%d: %s\n",x,List[x]);
								}
								break; */
							case GP2X_BUTTON_CLICK:
							case GP2X_BUTTON_A:
							case GP2X_BUTTON_X:
								if (PageNr < Lines)
								{
									PageNr++;
									NrOfChars = 0;
								}
								else
									if (Level < 36)
										GameState = GSNextStage;
									else
										GameState = GSCredits;
								break;
							case GP2X_BUTTON_SELECT:
								Mix_HaltMusic();
								GameState = GSTitleScreen;
								break;
							case GP2X_BUTTON_Y:
							case GP2X_BUTTON_B:
								if (Level < 36)
									GameState = GSNextStage;
								else
									GameState = GSCredits;
								break;
						}
					}
				}
				SDL_BlitSurface(BackGround,NULL,Buffer,NULL);
				Fairy.Draw();
				TextDelay++;
				if (TextDelay == 2)
				{
					if (NrOfChars < strlen(List[PageNr]))
						NrOfChars++;
					TextDelay = 0;
				}
				WriteText(Buffer,font,List[PageNr],NrOfChars,50,50,5,Color1);
				SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
                SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
                SDL_Flip(Screen);
			}
			SDL_FreeSurface(BackGround);
			break;
		default:
			GameState = GSNextStage;
			break;
	}
}

CPasswordSelector::CPasswordSelector()
{
	Image = IMG_Load("./graphics/select.png");
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY,SDL_MapRGB(Image->format,255,0,255));
	X = 0;
	Y = 0;
}

void CPasswordSelector::Draw()
{
	SDL_Rect DstRect;
	DstRect.x = XOffsetPassword + X * 36;
	DstRect.y = YOffsetPassword + Y * 36;
	DstRect.w = Image->w;
	DstRect.h = Image->h;
	SDL_BlitSurface(Image,NULL,Buffer,&DstRect);
}

void CPasswordSelector::MoveDown()
{
	if (Y < LetterRows - 2)
	{
		Y = Y + 1;
		if (GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}
	else
		if ((Y == LetterRows -2) && (X < LetterCols -2))
		{
			Y = Y +1;
			if (GlobalSoundEnabled && SoundEnabled)
				Mix_PlayChannel(-1,Sounds[SND_Menu],0);
		}
}

void CPasswordSelector::MoveLeft()
{
	if (X > 0)
	{
		X = X -1;
		if (GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}
}

void CPasswordSelector::MoveRight()
{
	if (Y < LetterRows -1)
		if (X < LetterCols -1)
		{
			X = X +1;
			if (GlobalSoundEnabled && SoundEnabled)
				Mix_PlayChannel(-1,Sounds[SND_Menu],0);
		}
	if (Y == LetterRows - 1)
		if (X < LetterCols -3)
		{
			X = X +1;
			if (GlobalSoundEnabled && SoundEnabled)
				Mix_PlayChannel(-1,Sounds[SND_Menu],0);
		}
}

void CPasswordSelector::MoveUp()
{
	if (Y > 0)
	{
		Y = Y - 1;
		if (GlobalSoundEnabled && SoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}
}

int CPasswordSelector::GetX()
{
	return X;
}

int CPasswordSelector::GetY()
{
	return Y;
}

CPasswordSelector::~CPasswordSelector()
{
	SDL_FreeSurface(Image);
}

int GetLevel(char *Password)
{
	int Result,Teller;
	Result = -1;
	bool Tmp = true;
	for (Teller = 0;Teller < 36;Teller++)
	{
		if (strcmp(Password,LevelPasswords[Teller]) ==0)
		{
			Result = Teller;
			break;
		}
	}
	return Result;
}

void PasswordEntry()
{
	SDL_Event Event;
	SDL_Surface* BackGround,*Text;
	SDL_Color Color1 = {0,0,0,0};
	SDL_Rect DstRect;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	int CharNr = 0;
	int LetterX,LetterY;
	char Password[5] = "    ";
	CPasswordSelector Selector;
	BackGround = IMG_Load("./graphics/paper.png");
	SDL_PollEvent(&Event);
	while (GameState == GSPasswordEntry)
	{
		while(SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
				/*	case GP2X_BUTTON_L:
						for (LetterY = 0;LetterY< LetterRows;LetterY++)
						{
							for (LetterX = 0; LetterX < LetterCols;LetterX++)
								printf("%s ",Letters[LetterY][LetterX]);
							printf("\n");
						}
						break; */
					case GP2X_BUTTON_SELECT:
					case GP2X_BUTTON_Y:
					case GP2X_BUTTON_B:
						GameState = GSTitleScreen;
						break;
					case GP2X_BUTTON_CLICK:
					case GP2X_BUTTON_X:
					case GP2X_BUTTON_A:
						Password[CharNr] = Letters[Selector.GetY()][Selector.GetX()][0];
						if (GlobalSoundEnabled && SoundEnabled)
							Mix_PlayChannel(-1,Sounds[SND_Select],0);
						CharNr++;
						if (CharNr > 3)
							CharNr = 0;
						break;
					case GP2X_BUTTON_UP:
						Selector.MoveUp();
						break;
					case GP2X_BUTTON_DOWN:
						Selector.MoveDown();
						break;
					case GP2X_BUTTON_LEFT:
						Selector.MoveLeft();
						break;
					case GP2X_BUTTON_RIGHT:
						Selector.MoveRight();
						break;
				/*	case GP2X_BUTTON_UPLEFT:
						Selector.MoveUp();
						Selector.MoveLeft();
						break;
					case GP2X_BUTTON_UPRIGHT:
						Selector.MoveUp();
						Selector.MoveRight();
						break;
					case GP2X_BUTTON_DOWNLEFT:
						Selector.MoveDown();
						Selector.MoveLeft();
						break;
					case GP2X_BUTTON_DOWNRIGHT:
						Selector.MoveDown();
						Selector.MoveRight();
						break;
						*/
				}
			}
		}
		SDL_BlitSurface(BackGround,NULL,Buffer,NULL);
		if (strlen(Password) > 0)
		{
			Text = TTF_RenderText_Solid(font,Password,Color1);
			DstRect.x = Buffer->w / 2 - Text->w / 2;
			DstRect.y = 37;
			DstRect.w = Text->w;
			DstRect.h = Text->h;
			SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
			SDL_FreeSurface(Text);
		}
		for (LetterY = 0 ;LetterY < LetterRows;LetterY++)
			for (LetterX = 0;LetterX < LetterCols;LetterX++)
			{
				if(Letters[LetterY][LetterX] != "0")
				{
					Text = TTF_RenderText_Solid(font,Letters[LetterY][LetterX],Color1);
					DstRect.x = XOffsetPassword + LetterX * 35;
					DstRect.y = YOffsetPassword + LetterY * 35;
					DstRect.w = Text->w;
					DstRect.h = Text->h;
					SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
					SDL_FreeSurface(Text);
				}
			}
		Selector.Draw();
		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
		if (Password[3] != ' ')
		{
			printf("Getlevel : %d\n",GetLevel(Password));
			SDL_Delay(250);
			if (GetLevel(Password) == -1)
			{
				Password[0] = ' ';
				Password[1] = ' ';
				Password[2] = ' ';
				Password[3] = ' ';
				if (GlobalSoundEnabled && SoundEnabled)
					Mix_PlayChannel(-1,Sounds[SND_Error],0);
			}
			else
				if(GetLevel(Password) == 0)
				{
					printf("gamestate = stageSelect\n");
					GameState = GSStageSelect;
				}
				else
				{
					Level = GetLevel(Password);
					GameState = GSOldManSpeaking;
					Mix_HaltMusic();
				}
		}
	}
	SDL_FreeSurface(BackGround);
}

CTitleScreenSelector::CTitleScreenSelector()
{
	Image=IMG_Load("./graphics/titleselector.png");
	SDL_SetColorKey(Image,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Image->format,255,0,255));
    Selection = 1;
}
void CTitleScreenSelector::MoveUp()
{

	if (Selection > 1)
	{
		Selection--;
		if(GlobalSoundEnabled && SoundEnabled) Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}

}
void CTitleScreenSelector::MoveDown()
{
	if (Selection < 4)
	{
		Selection++;
		if(GlobalSoundEnabled && SoundEnabled) Mix_PlayChannel(-1,Sounds[SND_Menu],0);
	}

}
void CTitleScreenSelector::Draw()
{
	SDL_Rect DstRect;
	switch (Selection)
	{
		case 1 : DstRect.y = 66;
				 break;
		case 2 : DstRect.y = 100;
				 break;
		case 3 : DstRect.y = 136;
				 break;
		case 4 : DstRect.y = 175;
				 break;
	}
	DstRect.x = 71;
	DstRect.w = Image->w;
	DstRect.h = Image->h;
	SDL_BlitSurface(Image,NULL,Buffer,&DstRect);
}

int CTitleScreenSelector::GetSelection()
{
	return Selection;
}

CTitleScreenSelector::~CTitleScreenSelector()
{
	SDL_FreeSurface(Image);
}


void TitleScreen()
{
	SDL_Surface *Background,*text;
	SDL_Event Event;
	bool RPressed=false;
	Uint32 OldTime;
	OldTime =0;
	char fps[100];
	SDL_Rect Rect;
	CTitleScreenSelector Selector;
	SDL_PollEvent(&Event);
	Background = IMG_Load("./graphics/title.png");
	if (GlobalSoundEnabled && MusicEnabled && (Mix_PlayingMusic() == 0))  Mix_PlayMusic(Music[MUS_Title],-1);
	while (GameState == GSTitleScreen)
	{
		while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONUP)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_R:
                        RPressed = false;
                        break;
                    case GP2X_BUTTON_LEFT:
                        if (RPressed)
                            if(StartScreenX - 2 >= 0)
                                StartScreenX -=2;
                        break;
                    case GP2X_BUTTON_RIGHT:
                        if (RPressed)
                            if(StartScreenX + 2 + Buffer->w <= Screen->w)
                                StartScreenX +=2;
                        break;
                    case GP2X_BUTTON_UP:
                        if(RPressed)
                            if(StartScreenY - 2 >=0)
                                StartScreenY -=2;
                        break;
                    case GP2X_BUTTON_DOWN:
                        if(RPressed)
                            if(StartScreenY + 2 + Buffer->h <= Screen->h)
                                StartScreenY +=2;
                        break;
                }

            if (Event.type == SDL_JOYBUTTONDOWN)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_R:
                        RPressed = true;
                        break;
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					case GP2X_BUTTON_SELECT:
						GameState = GSQuit;
						break;
					case GP2X_BUTTON_UP :
						Selector.MoveUp();
						break;
					case GP2X_BUTTON_DOWN :
						Selector.MoveDown();
						break;
					case GP2X_BUTTON_CLICK:
					case GP2X_BUTTON_X:
					case GP2X_BUTTON_START:
					case GP2X_BUTTON_A:
						if (GlobalSoundEnabled & SoundEnabled) Mix_PlayChannel(-1,Sounds[SND_Select],0);
						switch (Selector.GetSelection())
						{
							case 1:
								Level = 0;
								GameState = GSOldManSpeaking;
								break;
							case 2:
								GameState = GSPasswordEntry;
								break;
							case 3:
								GameState = GSOptions;
								break;
							case 4:
								GameState = GSCredits;
								break;
						}
     			}
        }
        //sprintf(fps,"fps:%d",1000 / (SDL_GetTicks() - OldTime));
        //OldTime=SDL_GetTicks();
        //text = TTF_RenderText_Solid(font,fps,Color);
		SDL_BlitSurface(Background,NULL,Buffer,NULL);
		//SDL_BlitSurface(text,NULL,Buffer,NULL);
		//SDL_FreeSurface(text);
		Selector.Draw();
		Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
	}
	SDL_FreeSurface(Background);
}

void Intro()
{
    SDL_Surface *Pict1,*Pict2,*Pict3,*Pict4,*Pict5,*Text;
    SDL_Rect DstRect;
    SDL_Event Event;
    int Pictnr,x,y;
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	x = 320;
	y = 240;
	Pictnr = 1;
	Pict1 = SDL_DisplayFormat(Screen);
	Pict2 = IMG_Load("./graphics/sdlpowered.png");
	Pict3 = IMG_Load("./graphics/willemssoft.png");
	Pict4 = IMG_Load("./graphics/title.png");
	Pict5 = SDL_DisplayFormat(Screen);
	SDL_SetColorKey(Pict5,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Pict5->format,255,0,255));
	SDL_FillRect(Pict1,NULL,SDL_MapRGB(Pict1->format,0,0,0));
    SDL_PollEvent(&Event);
    while (GameState == GSIntro)
    {
    	while(SDL_PollEvent(&Event))
    	{
        	switch(Event.type)
        	{
       			case SDL_JOYBUTTONDOWN :
                    if (Event.jbutton.button == GP2X_BUTTON_SELECT)
                    {
                        GameState = GSQuit;
                    }
                    else
                    {
                        GameState = GSTitleScreen;
                    }
        	}
        }
        switch(Pictnr)
        {
        	case 1: SDL_BlitSurface(Pict1,NULL,Buffer,NULL);
        			SDL_BlitSurface(Pict2,NULL,Pict5,NULL);
        			break;
        	case 2: SDL_BlitSurface(Pict2,NULL,Buffer,NULL);
        			SDL_BlitSurface(Pict3,NULL,Pict5,NULL);
        			break;
        	case 3: SDL_BlitSurface(Pict3,NULL,Buffer,NULL);
        			SDL_BlitSurface(Pict4,NULL,Pict5,NULL);
        			break;

        }
        DstRect.x = (int) 160 - x/2;
        DstRect.y = (int) 120 - y/2;
        DstRect.w = x;
        DstRect.h = y;
        SDL_FillRect(Pict5,&DstRect,SDL_MapRGB(Pict5->format,255,0,255));
        SDL_BlitSurface(Pict5,NULL,Buffer,NULL);
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        x -= 20;
        if (y >0)
        {
        	y -= 15;
        }
        else
        {
        	Pictnr++;
        	if (Pictnr == 4)
        	{
				GameState = GSTitleScreen;
        	}
        	else
        	SDL_Delay(2000);
        	x = 320;
        	y = 120;
        }

    }
    SDL_FreeSurface(Pict1);
    SDL_FreeSurface(Pict2);
    SDL_FreeSurface(Pict3);
    SDL_FreeSurface(Pict4);
    SDL_FreeSurface(Pict5);
    SDL_PollEvent(&Event);
}

void Credits()
{
	char *Tekst = "Puzzle Land is programmed by\nWillems Davy. Graphics are made\nusing Paint Shop Pro 7. A Big thanks\ngoes out to Yann R. Fernandez for\ncreating some of the graphics.\nPuzzle Land is a remake of\nDaedalian opus for gameboy by\nVic Tokai Inc. 1990\n\nWillems Soft 2006";
	SDL_Event Event;
	SDL_Surface* BackGround;
	SDL_Color Color1 = {0,0,0,0};
	int NrOfChars = 0;
	int TextDelay = 0;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	BackGround = IMG_Load("./graphics/paper.png");
	while (GameState == GSCredits)
	{
		while (SDL_PollEvent(&Event))
		{
			if(Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					default:
						GameState = GSTitleScreen;
						break;
				}
			}
		}
		if (NrOfChars < strlen(Tekst))
			NrOfChars++;
		SDL_BlitSurface(BackGround,NULL,Buffer,NULL);
		TTF_SetFontStyle(font,TTF_STYLE_UNDERLINE);
		WriteText(Buffer,font,"CREDITS",7,Buffer-> w / 2 - 32,20,0,Color1);
		TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
		WriteText(Buffer,font,Tekst,NrOfChars,30,40,0,Color1);
		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
	}
	SDL_FreeSurface(BackGround);
}

//intialization & Finalasation stuff -----------------------------------
void LoadImages()
{
	int teller;
	char FileName[FILENAME_MAX];
	for (teller=0;teller<BlockCount;teller++)
	{
		sprintf(FileName,"./graphics/block%d.png",teller+1);
		BlockImages[teller] = IMG_Load(FileName);
	}
	for (teller=0;teller<BorderCount;teller++)
	{
		sprintf(FileName,"./graphics/border%d.png",teller+1);
		BorderImages[teller] = IMG_Load(FileName);
		SDL_SetColorKey(BorderImages[teller],SDL_RLEACCEL | SDL_SRCCOLORKEY,SDL_MapRGB(BorderImages[teller]->format,255,0,255));
	}
    RoomBackground = IMG_Load("./graphics/roombackground.png");
 	StageClearKader = IMG_Load("./graphics/stageclearkader.png");
 	SDL_SetColorKey(StageClearKader,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(StageClearKader->format,255,0,255));
 	SDL_SetAlpha(StageClearKader,SDL_SRCALPHA,225);
 	Bridge = IMG_Load("./graphics/bridge.png");
 	SDL_SetColorKey(Bridge,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Bridge->format,255,0,255));
}

void UnloadImages()
{
	int teller;
	for (teller=0;teller<BlockCount;teller++)
	{
		SDL_FreeSurface(BlockImages[teller]);
	}
	for (teller=0;teller<BorderCount;teller++)
	{
		SDL_FreeSurface(BorderImages[teller]);
	}
	SDL_FreeSurface(RoomBackground);
	SDL_FreeSurface(StageClearKader);
	SDL_FreeSurface(Bridge);
}

void UnloadSounds()
{
 	int teller;
 	for(teller=0;teller < NROFSOUNDS;teller++)
 	{
		Mix_FreeChunk(Sounds[teller]);
 	}
}

void LoadSounds()
{
	Sounds[SND_RotateBlock] = Mix_LoadWAV("./sounds/rotate.wav");
 	Sounds[SND_DropBlock] = Mix_LoadWAV("./sounds/drop.wav");
 	Sounds[SND_FlipBlock] = Mix_LoadWAV("./sounds/rotate.wav");
 	Sounds[SND_PickupBlock] = Mix_LoadWAV("./sounds/pickup.wav");
 	Sounds[SND_Error] = Mix_LoadWAV("./sounds/error.wav");
 	Sounds[SND_Menu] = Mix_LoadWAV("./sounds/menu.wav");
 	Sounds[SND_Select] = Mix_LoadWAV("./sounds/select.wav");
 	Sounds[SND_Bridge] = Mix_LoadWAV("./sounds/bridge.wav");
 	Sounds[SND_StageEnd] = Mix_LoadWAV("./sounds/stageend.wav");
 	Sounds[SND_Elf] = Mix_LoadWAV("./sounds/elf.wav");
}

void LoadMusic()
{
 	Music[MUS_Title] = Mix_LoadMUS("./music/title.ogg");
 	Music[MUS_Game1] = Mix_LoadMUS("./music/game1.ogg");
 	Music[MUS_Game2] = Mix_LoadMUS("./music/game2.ogg");
 	Music[MUS_Oldman] = Mix_LoadMUS("./music/oldman.ogg");
 	Music[MUS_Stage] = Mix_LoadMUS("./music/stage.ogg");
}


void UnloadMusic()
{
 	int teller;
    Mix_HaltMusic();
	for(teller=0;teller<NROFMUSIC;teller++)
	{
		Mix_FreeMusic(Music[teller]);
	}
}

int main(int argc, char **argv)
{
    SDL_Rect Rect;
	if (SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO ) == 0)
	{
		printf("SDL Succesfully initialized\n");
		if(SDL_GP2X_GetPhysicalScreenSize(&Rect) == 0)
            Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_SWSURFACE );
        else
        {
            TvOutMode = true;
            Screen = SDL_SetVideoMode( 360, 288,16, SDL_SWSURFACE );
        }
		if(Screen)
		{
		    Buffer = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			printf("Succesfully Set %dx%dx16\n",WINDOW_WIDTH,WINDOW_HEIGHT);
			SDL_ShowCursor(SDL_DISABLE);
			Joystick = SDL_JoystickOpen(0);
			if(Joystick)
			{
				if (Mix_OpenAudio(11025,AUDIO_S16,MIX_DEFAULT_CHANNELS,64) < 0)
				{
					GlobalSoundEnabled = false;
					printf("Failed to initialise sound!\n");
					printf("%s\n",Mix_GetError());
				}
				else
				{
			 		printf("Audio Succesfully initialised!\n");
				}
		    	LoadSounds();
				LoadMusic();
				LoadSettings();
				LoadImages();
				if (TTF_Init() == 0)
				{
					printf("Succesfully initialized TTF\n");
					font = TTF_OpenFont("font.ttf",25);
					if (font)
					{
						printf("Succesfully Loaded font\n");
						TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
						while (GameState != GSQuit)
						{
							printf("Start Main Switch\n");
							switch(GameState)
							{
								case GSIntro :
									Intro();
									break;
								case GSTitleScreen :
									TitleScreen();
									break;
								case GSOldManSpeaking :
									OldManSpeaking();
									break;
								case GSPasswordEntry :
									PasswordEntry();
									break;
								case GSCredits :
									Credits();
									break;
								case GSOptions :
									Options();
									break;
								case GSGame :
									printf("Starting Game()\n");
									Game();
									break;
								case GSStageClear:
									StageClear();
									break;
								case GSNextStage:
									if (Level ==0)
										NextStageLevel0();
									else
										NextStageLevel1to35();
									printf("Return from Next stage Level 1 - 35\n");
									break;
								case GSStageSelect:
									StageSelect();
									break;
							}
							printf("Main Switch ended\n");
						}
						TTF_CloseFont(font);
				    	font=NULL;
					}
					else
					{
						printf("Failed to Load font\n");
					}
					TTF_Quit();
				}
				else
				{
					printf("Failed to initialize TTF\n");
				}
				SDL_FreeSurface(Screen);
				Screen=NULL;
				Mix_CloseAudio();
				UnloadSounds();
				UnloadMusic();
				UnloadImages();
				SaveSettings();
				if(SDL_JoystickOpened(0))
    				SDL_JoystickClose(Joystick);
			}
			else
			{
				printf("Failed to open joystick(0)!\n");
			}
		}
		else
		{
			printf("Failed to Set Videomode %dx%dx32\n",WINDOW_WIDTH, WINDOW_HEIGHT);
		}
		SDL_Quit();
	}
	else
	{
		printf("Couldn't initialise SDL!\n");
	}
	system("sync");
	//chdir("/usr/gp2x");
	//execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
	return 0;
}
